@@ -33,8 +33,8 @@ static int runuritoid(int argc, char **argv);
 static int procinform(const char *dbname, const char *outfile);
 static int procsearch(const char *dbname, CBMAP *params, const char *outfile,
                       const char *myurl, const char *mylabel, int rateuri, int mergemethod,
-                      int scoreexpr, int searchmax, int wwidth, int hwidth, int awidth,
-                      int scancheck, int smlrvnum);
+                      int scoreexpr, int searchmax, int wildmax,
+                      int wwidth, int hwidth, int awidth, int scancheck, int smlrvnum);
 static int procgetdoc(const char *dbname, const char *outfile, int id, const char *uri);
 static int procgetdocattr(const char *dbname, const char *outfile,
                           int id, const char *uri, const char *attr);
@@ -105,8 +105,8 @@ static int runsearch(int argc, char **argv){
   const char *dbname, *outfile;
   char *pbuf, *myurl, *mylabel;
   int rv, size, scoreexpr, rateuri, mergemethod, searchmax;
-  int wwidth, hwidth, awidth, scancheck, smlrvnum;
-  if(argc < 16) usage();
+  int wildmax, wwidth, hwidth, awidth, scancheck, smlrvnum;
+  if(argc < 17) usage();
   dbname = argv[2];
   outfile = argv[3];
   pbuf = cbbasedecode(argv[4], &size);
@@ -117,14 +117,15 @@ static int runsearch(int argc, char **argv){
   mergemethod = atoi(argv[8]);
   scoreexpr = atoi(argv[9]);
   searchmax = atoi(argv[10]);
-  wwidth = atoi(argv[11]);
-  hwidth = atoi(argv[12]);
-  awidth = atoi(argv[13]);
-  scancheck = atoi(argv[14]);
-  smlrvnum = atoi(argv[15]);
-  if(searchmax < 0 || wwidth < 0 || hwidth < 0 || awidth < 0) usage();
+  wildmax = atoi(argv[11]);
+  wwidth = atoi(argv[12]);
+  hwidth = atoi(argv[13]);
+  awidth = atoi(argv[14]);
+  scancheck = atoi(argv[15]);
+  smlrvnum = atoi(argv[16]);
+  if(searchmax < 0 || wildmax < 0 || wwidth < 0 || hwidth < 0 || awidth < 0) usage();
   rv = procsearch(dbname, params, outfile, myurl, mylabel, rateuri, mergemethod,
-                  scoreexpr, searchmax, wwidth, hwidth, awidth, scancheck, smlrvnum);
+                  scoreexpr, searchmax, wildmax, wwidth, hwidth, awidth, scancheck, smlrvnum);
   free(mylabel);
   free(myurl);
   cbmapclose(params);
@@ -216,11 +217,10 @@ static int procinform(const char *dbname, const char *outfile){
 /* perform the search command */
 static int procsearch(const char *dbname, CBMAP *params, const char *outfile,
                       const char *myurl, const char *mylabel, int rateuri, int mergemethod,
-                      int scoreexpr, int searchmax, int wwidth, int hwidth, int awidth,
-                      int scancheck, int smlrvnum){
+                      int scoreexpr, int searchmax, int wildmax,
+                      int wwidth, int hwidth, int awidth, int scancheck, int smlrvnum){
   RESMAP *resmap;
   RESDOC **resdocs, *resdoc;
-
   ESTDB *db;
   ESTCOND *cond;
   ESTDOC *doc;
@@ -228,13 +228,11 @@ static int procsearch(const char *dbname, CBMAP *params, const char *outfile,
   const CBLIST *texts;
   CBLIST *words, *list;
   CBDATUM *datum;
-
-  const char *bordstr, *tmp, *kbuf, *vbuf, *phrase, *order;
+  const char *bordstr, *tmp, *kbuf, *vbuf, *phrase, *order, *distinct;
   char name[NUMBUFSIZ], *snippet;
-  int i, j, ecode, err, max, skip, num, ksiz, vsiz, lmax, cnt, score, down, dnum, wnum, hnum;
-  int end, *res, rnum, id;
+  int i, j, ecode, err, max, skip, num, ksiz, vsiz, lmax, cnt, score, down, dnum, wnum;
+  int end, *res, rnum, hnum, id;
   double curtime, itime, weight, fsiz;
-
   if(!(db = est_db_open(dbname, ESTDBREADER | ESTDBNOLCK, &ecode))) return 1;
   err = FALSE;
   bordstr = est_border_str();
@@ -261,6 +259,8 @@ static int procsearch(const char *dbname, CBMAP *params, const char *outfile,
     est_cond_set_options(cond, num);
   if((tmp = cbmapget(params, "auxiliary", -1, NULL)) != NULL)
     est_cond_set_auxiliary(cond, atoi(tmp));
+  if((tmp = cbmapget(params, "distinct", -1, NULL)) != NULL && tmp[0] != '\0')
+    est_cond_set_order(cond, tmp);
   if((tmp = cbmapget(params, "wwidth", -1, NULL)) != NULL) wwidth = atoi(tmp);
   if((tmp = cbmapget(params, "hwidth", -1, NULL)) != NULL && (num = atoi(tmp)) >= 0)
     hwidth = num;
@@ -351,7 +351,6 @@ static int procsearch(const char *dbname, CBMAP *params, const char *outfile,
   dnum = est_db_doc_num(db);
   wnum = est_db_word_num(db);
   fsiz = est_db_size(db);
-  hnum = (tmp = cbmapget(hints, "", 0, NULL)) ? atoi(tmp) : 0;
   end = max + skip;
   curtime = est_gettimeofday() - curtime;
   datum = cbdatumopen(NULL, -1);
@@ -380,7 +379,8 @@ static int procsearch(const char *dbname, CBMAP *params, const char *outfile,
   order = est_cond_order(cond);
   if(!order && phrase && cbstrfwmatch(phrase, ESTOPSIMILAR))
     order = DATTRNDSCORE " " ESTORDNUMD;
-  resdocs = resmap_list(resmap, &rnum, order);
+  distinct = est_cond_distinct(cond);
+  resdocs = resmap_list(resmap, &rnum, order, distinct);
   for(i = skip; i < rnum && i < end; i++){
     resdoc = resdocs[i];
     if(!resdoc->doc) continue;
