@@ -85,7 +85,8 @@ static int procinform(const char *nurl);
 static int procsync(const char *nurl);
 static int procoptimize(const char *nurl);
 static int procsearch(const char *nurl, const char *phrase, const CBLIST *attrs, const char *ord,
-                      int max, int sk, int aux, int opts, int depth, int mask, double ec);
+                      int max, int sk, int aux, int opts, int depth, int mask, double ec,
+                      const char *dis);
 static int proclist(const char *nurl);
 static int procsetuser(const char *nurl, const char *name, int mode);
 static int procsetlink(const char *nurl, const char *url, const char *label, int credit);
@@ -164,7 +165,7 @@ static void usage(void){
           " nurl\n", g_progname);
   fprintf(stderr, "  %s search [-proxy host port] [-tout num] [-auth user pass]"
           " [-vu|-vx] [-ec rn] [-sf] [-attr expr] [-ord expr] [-max num] [-sk num] [-aux num]"
-          " [-dpt num] [-mask num] nurl [phrase]\n", g_progname);
+          " [-dis name] [-dpt num] [-mask num] nurl [phrase]\n", g_progname);
   fprintf(stderr, "  %s list [-proxy host port] [-tout num] [-auth user pass]"
           " nurl\n", g_progname);
   fprintf(stderr, "  %s setuser [-proxy host port] [-tout num] [-auth user pass]"
@@ -548,11 +549,12 @@ static int runoptimize(int argc, char **argv){
 static int runsearch(int argc, char **argv){
   CBDATUM *pbuf;
   CBLIST *attrs;
-  char *nurl, *ord;
+  char *nurl, *ord, *dis;
   int i, max, sk, aux, opts, depth, mask, rv;
   double ec;
   nurl = NULL;
   ord = NULL;
+  dis = NULL;
   max = SEARCHMAX;
   sk = 0;
   aux = SEARCHAUX;
@@ -603,6 +605,9 @@ static int runsearch(int argc, char **argv){
       } else if(!strcmp(argv[i], "-aux")){
         if(++i >= argc) usage();
         aux = atoi(argv[i]);
+      } else if(!strcmp(argv[i], "-dis")){
+        if(++i >= argc) usage();
+        dis = argv[i];
       } else if(!strcmp(argv[i], "-dpt")){
         if(++i >= argc) usage();
         depth = atoi(argv[i]);
@@ -620,7 +625,7 @@ static int runsearch(int argc, char **argv){
     }
   }
   if(!nurl || depth < 0) usage();
-  rv = procsearch(nurl, cbdatumptr(pbuf), attrs, ord, max, sk, aux, opts, depth, mask, ec);
+  rv = procsearch(nurl, cbdatumptr(pbuf), attrs, ord, max, sk, aux, opts, depth, mask, ec, dis);
   return rv;
 }
 
@@ -804,7 +809,7 @@ static void xmlprintf(const char *format, ...){
       cbuf[0] = '%';
       cblen = 1;
       format++;
-      while(strchr("0123456789 .+-", *format) && *format != '\0' && cblen < 31){
+      while(strchr("0123456789 .+-", *format) && *format != '\0' && cblen < sizeof(cbuf) - 1){
         cbuf[cblen++] = *format;
         format++;
       }
@@ -1118,7 +1123,8 @@ static int procoptimize(const char *nurl){
 
 /* perform the inform command */
 static int procsearch(const char *nurl, const char *phrase, const CBLIST *attrs, const char *ord,
-                      int max, int sk, int aux, int opts, int depth, int mask, double ec){
+                      int max, int sk, int aux, int opts, int depth, int mask, double ec,
+                      const char *dis){
   ESTNODE *node;
   ESTNODERES *nres;
   ESTRESDOC *rdoc, **sdocs;
@@ -1151,6 +1157,7 @@ static int procsearch(const char *nurl, const char *phrase, const CBLIST *attrs,
   if(sk > 0) est_cond_set_skip(cond, sk);
   est_cond_set_options(cond, opts);
   est_cond_set_auxiliary(cond, aux);
+  if(dis) est_cond_set_distinct(cond, dis);
   est_cond_set_mask(cond, mask);
   if((nres = est_node_search(node, cond, depth)) != NULL){
     hints = est_noderes_hints(nres);
