@@ -392,6 +392,17 @@ void est_mtdb_set_ecode(ESTMTDB *db, int ecode){
 }
 
 
+/* Check whether an option is set. */
+int est_mtdb_check_option(ESTMTDB *db, int option){
+  int rv;
+  assert(db);
+  if(!est_mtdb_lock(db)) return -1;
+  rv = est_db_check_option(db->db, option);
+  est_mtdb_unlock(db);
+  return rv;
+}
+
+
 /* Get the inode number of a database. */
 int est_mtdb_inode(ESTMTDB *db){
   int rv;
@@ -707,7 +718,7 @@ void est_mtdb_charge_rescc(ESTMTDB *db, int max){
 }
 
 
-/* Get list of words in the result cache. */
+/* Get a list of words in the result cache. */
 CBLIST *est_mtdb_list_rescc(ESTMTDB *db){
   CBLIST *rv;
   assert(db);
@@ -718,6 +729,28 @@ CBLIST *est_mtdb_list_rescc(ESTMTDB *db){
 }
 
 
+/* Get the number of pseudo documents in a database. */
+int est_mtdb_pseudo_doc_num(ESTMTDB *db){
+  int rv;
+  assert(db);
+  if(!est_mtdb_lock(db)) return 0;
+  rv = est_db_pseudo_doc_num(db->db);
+  est_mtdb_unlock(db);
+  return rv;
+}
+
+
+/* Get a list of expressions of attribute indexes of a database. */
+CBLIST *est_mtdb_attr_index_exprs(ESTMTDB *db){
+  CBLIST *rv;
+  assert(db);
+  if(!est_mtdb_lock(db)) return 0;
+  rv = est_db_attr_index_exprs(db->db);
+  est_mtdb_unlock(db);
+  return rv;
+}
+
+
 /* Interrupt long time processing. */
 void est_mtdb_interrupt(ESTMTDB *db){
   assert(db);
