@@ -1,6 +1,6 @@
 /*************************************************************************************************
  * The command line interface of the MT-safe API
- *                                                      Copyright (C) 2004-2005 Mikio Hirabayashi
+ *                                                      Copyright (C) 2004-2006 Mikio Hirabayashi
  * This file is part of Hyper Estraier.
  * Hyper Estraier is free software; you can redistribute it and/or modify it under the terms of
  * the GNU Lesser General Public License as published by the Free Software Foundation; either
@@ -48,12 +48,14 @@ int g_sigterm = FALSE;                   /* flag for termination signal */
 int main(int argc, char **argv);
 static void printferror(const char *format, ...);
 static void printfinfo(const char *format, ...);
-static void dbinform(const char *msg);
+static void dbinform(const char *message, void *opaque);
 static void setsignals(void);
 static void sigtermhandler(int num);
 static void usage(void);
 static int runwicked(int argc, char **argv);
+static int runregression(int argc, char **argv);
 static int procwicked(const char *dbname, int dnum, int tnum);
+static int procregression(const char *dbname);
 static void *mtwkfunc(void *mission);
 static ESTDOC *est_doc_new_from_chaos(int cnum, int snum, int mode);
 static char *est_random_str(int cnum, int mode);
@@ -64,7 +66,7 @@ int main(int argc, char **argv){
   const char *tmp;
   int rv;
   if((tmp = getenv("ESTDBGFD")) != NULL) dpdbgfd = atoi(tmp);
-  cbstdiobin();
+  est_proc_env_reset();
   g_progname = argv[0];
   g_sigterm = FALSE;
   if(argc < 2) usage();
@@ -72,6 +74,9 @@ int main(int argc, char **argv){
   if(!strcmp(argv[1], "wicked")){
     setsignals();
     rv = runwicked(argc, argv);
+  } else if(!strcmp(argv[1], "regression")){
+    setsignals();
+    rv = runregression(argc, argv);
   } else {
     usage();
   }
@@ -107,8 +112,8 @@ static void printfinfo(const char *format, ...){
 
 
 /* callback function for database events */
-static void dbinform(const char *msg){
-  printfinfo("%s", msg);
+static void dbinform(const char *message, void *opaque){
+  printfinfo("%s", message);
 }
 
 
@@ -141,6 +146,7 @@ static void usage(void){
   fprintf(stderr, "\n");
   fprintf(stderr, "usage:\n");
   fprintf(stderr, "  %s wicked db dnum tnum\n", g_progname);
+  fprintf(stderr, "  %s regression db\n", g_progname);
   fprintf(stderr, "\n");
   exit(1);
 }
@@ -174,6 +180,26 @@ static int runwicked(int argc, char **argv){
 }
 
 
+/* parse arguments of the regression command */
+static int runregression(int argc, char **argv){
+  char *dbname;
+  int i, rv;
+  dbname = NULL;
+  for(i = 2; i < argc; i++){
+    if(!dbname && argv[i][0] == '-'){
+      usage();
+    } else if(!dbname){
+      dbname = argv[i];
+    } else {
+      usage();
+    }
+  }
+  if(!dbname) usage();
+  rv = procregression(dbname);
+  return rv;
+}
+
+
 /* perform the wicked command */
 static int procwicked(const char *dbname, int dnum, int tnum){
   ESTMTDB *dbs[BRANCHDBNUM];
@@ -195,7 +221,7 @@ static int procwicked(const char *dbname, int dnum, int tnum){
       free(name);
       return -1;
     }
-    est_mtdb_set_informer(dbs[i], dbinform);
+    est_mtdb_set_informer(dbs[i], dbinform, NULL);
     est_mtdb_set_cache_size(dbs[i], 1024 * 1024 * 32, 1024, 256, 64);
     est_mtdb_set_special_cache(dbs[i], ESTDATTRURI, 128);
     free(name);
@@ -246,6 +272,126 @@ static int procwicked(const char *dbname, int dnum, int tnum){
 }
 
 
+/* perform the regression command */
+static int procregression(const char *dbname){
+  ESTMTDB *dbs[10], *db, *tdbs[5];
+  ESTDOC *doc;
+  ESTCOND *cond;
+  CBMAP *hints;
+  char wbuf[URIBUFSIZ];
+  int i, j, ecode, err, *res, rnum;
+  time_t curtime;
+  curtime = time(NULL);
+  printfinfo("# opening databases");
+  for(i = 0; i < 10; i++){
+    sprintf(wbuf, "%s-%08d", dbname, i + 1);
+    if(!(dbs[i] = est_mtdb_open(wbuf, ESTDBWRITER | ESTDBCREAT | ESTDBTRUNC, &ecode))){
+      printferror("%s: %s", wbuf, est_err_msg(est_mtdb_error(dbs[i])));
+      return 1;
+    }
+    est_mtdb_set_informer(dbs[i], dbinform, NULL);
+  }
+  err = FALSE;
+  printfinfo("# registering documents");
+  for(i = 0; i < 1000 && !g_sigterm; i++){
+    doc = est_doc_new();
+    sprintf(wbuf, "%d", rand() % (i + 100));
+    est_doc_add_attr(doc, ESTDATTRURI, wbuf);
+    for(j = 0; j < 100; j++){
+      sprintf(wbuf, "%c%c", 'a' + rand() % 6, 'a' + rand() % 6);
+      est_doc_add_text(doc, wbuf);
+    }
+    db = dbs[rand()%10];
+    if(!est_mtdb_put_doc(db, doc, ESTPDCLEAN)){
+      printferror("%s: %s", est_mtdb_name(db), est_err_msg(ecode));
+      err = TRUE;
+    }
+    est_doc_delete(doc);
+  }
+  printfinfo("# flushing half of databases");
+  for(i = 0; i < 10; i += 2){
+    if(!est_mtdb_flush(dbs[i], -1)){
+      printferror("%s: %s", est_mtdb_name(dbs[i]), est_err_msg(ecode));
+      err = TRUE;
+    }
+  }
+  printfinfo("# searching databases");
+  for(i = 0; i < 1000 && !g_sigterm; i++){
+    cond = est_cond_new();
+    sprintf(wbuf, "%c%c", 'a' + rand() % 6, 'a' + rand() % 6);
+    est_cond_set_phrase(cond, wbuf);
+    switch(rand() % 5){
+    case 0:
+      est_cond_set_order(cond, ESTORDIDA);
+      break;
+    case 1:
+      est_cond_set_order(cond, ESTORDSCA);
+      break;
+    case 2:
+      est_cond_set_order(cond, ESTDATTRURI " " ESTORDSTRA);
+      break;
+    }
+    hints = cbmapopen();
+    res = est_mtdb_search_meta(dbs, 10, cond, &rnum, hints);
+    for(j = 0; j < rnum; j += 2){
+      if(res[j] < 0 || res[j] >= 10){
+        printferror("%s: search result is invalid", dbname);
+        err = TRUE;
+        break;
+      }
+    }
+    free(res);
+    cbmapclose(hints);
+    est_cond_delete(cond);
+  }
+  printfinfo("# searching certainly same databases");
+  tdbs[0] = dbs[0];
+  tdbs[1] = dbs[1];
+  tdbs[2] = dbs[2];
+  tdbs[3] = dbs[0];
+  tdbs[4] = dbs[1];
+  for(i = 0; i < 100 && !g_sigterm; i++){
+    cond = est_cond_new();
+    sprintf(wbuf, "%c%c", 'a' + rand() % 6, 'a' + rand() % 6);
+    est_cond_set_phrase(cond, wbuf);
+    switch(rand() % 5){
+    case 0:
+      est_cond_set_order(cond, ESTORDIDA);
+      break;
+    case 1:
+      est_cond_set_order(cond, ESTORDSCA);
+      break;
+    case 2:
+      est_cond_set_order(cond, ESTDATTRURI " " ESTORDSTRA);
+      break;
+    }
+    hints = cbmapopen();
+    res = est_mtdb_search_meta(tdbs, 5, cond, &rnum, hints);
+    for(j = 0; j < rnum; j += 2){
+      if(res[j] < 0 || res[j] >= 5){
+        printferror("%s: search result is invalid", dbname);
+        err = TRUE;
+        break;
+      }
+    }
+    free(res);
+    cbmapclose(hints);
+    est_cond_delete(cond);
+  }
+  printfinfo("# closing databases");
+  for(i = 0; i < 10; i++){
+    if(!est_mtdb_close(dbs[i], &ecode)){
+      printferror("%s: %s", est_mtdb_name(dbs[i]), est_err_msg(ecode));
+      err = TRUE;
+    }
+  }
+  curtime = time(NULL) - curtime;
+  if(!err) printfinfo("# finished successfully: elapsed time: %dh %dm %ds",
+                      (int)(curtime / 3600), (int)((curtime / 60) % 60), (int)(curtime % 60));
+  return err ? 1 : 0;
+}
+
+
 /* Thread function for wicked command */
 static void *mtwkfunc(void *mission){
   ESTMTDB *db, **others;
@@ -333,9 +479,9 @@ static void *mtwkfunc(void *mission){
       doc = est_doc_new_from_chaos(100, 3, est_random() < 0.5 ? RD_EURO : RD_RAND);
       if(est_random() < 0.2){
         sprintf(uri, "file:///tmp/wicked-%08d-%05d.est",
-                (int)(est_random() * INT_MAX) % (i + 1) + 1, getpid());
+                (int)(est_random() * INT_MAX) % (i + 1) + 1, (int)getpid());
       } else {
-        sprintf(uri, "file:///tmp/wicked-%08d-%05d.est", i + 1, getpid());
+        sprintf(uri, "file:///tmp/wicked-%08d-%05d.est", i + 1, (int)getpid());
       }
       est_doc_add_attr(doc, ESTDATTRURI, uri);
       if(!est_mtdb_put_doc(db, doc, est_random() < 0.5 ? ESTPDCLEAN : 0)) err = TRUE;
