@@ -38,7 +38,7 @@
 #define ESTWWIDTHDEF   480               /* default of whole width of a snippet */
 #define ESTHWIDTHDEF   96                /* default of head width of a snippet */
 #define ESTAWIDTHDEF   96                /* default of around width of a snippet */
-#define ESTDNHOLDSEC   30               /* holding time of domain names */
+#define ESTDNHOLDSEC   300               /* holding time of domain names */
 #define ESTDNHOLDNUM   4096              /* holding number of domain names */
 
 typedef struct {                         /* type of structure for interaction of a URL */
@@ -1076,7 +1076,7 @@ ESTNODERES *est_node_search(ESTNODE *node, ESTCOND *cond, int depth){
   CBMAP *resheads;
   CBLIST *reqheads;
   CBDATUM *reqbody, *resbody;
-  const char *kbuf, *phrase, *order, *cbuf;
+  const char *kbuf, *phrase, *order, *distinct, *cbuf;
   char buf[ESTPATHBUFSIZ], *vbuf, *ptr, *pv, *ep;
   int i, rescode, err, max, plen, part, end;
   assert(node && cond && depth >= 0);
@@ -1114,6 +1114,8 @@ ESTNODERES *est_node_search(ESTNODE *node, ESTCOND *cond, int depth){
   if(cbdatumsize(reqbody) > 0) cbdatumcat(reqbody, "&", 1);
   est_datum_printf(reqbody, "options=%d", est_cond_options(cond));
   est_datum_printf(reqbody, "&auxiliary=%d", est_cond_auxiliary(cond));
+  if((distinct = est_cond_distinct(cond)) != NULL)
+    est_datum_printf(reqbody, "&distinct=%?", distinct);
   est_datum_printf(reqbody, "&depth=%d", depth);
   est_datum_printf(reqbody, "&wwidth=%d", node->wwidth);
   est_datum_printf(reqbody, "&hwidth=%d", node->hwidth);
