@@ -1,6 +1,6 @@
 /*************************************************************************************************
  * The node API of Hyper Estraier
- *                                                      Copyright (C) 2004-2005 Mikio Hirabayashi
+ *                                                      Copyright (C) 2004-2006 Mikio Hirabayashi
  * This file is part of Hyper Estraier.
  * Hyper Estraier is free software; you can redistribute it and/or modify it under the terms of
  * the GNU Lesser General Public License as published by the Free Software Foundation; either
@@ -56,7 +56,7 @@ typedef struct _ESTRESDOC {              /* type of structure for a document in
   const char *uri;                       /* URI */
   CBMAP *attrs;                          /* attributes */
   char *snippet;                         /* snippet */
-  CBMAP *kwords;                         /* keyword vector */
+  CBMAP *kwords;                         /* keywords */
   double similarity;                     /* similarity to the parent document */
   struct _ESTRESDOC **children;          /* child documents */
   int cnum;                              /* number of child documents */
@@ -70,6 +70,7 @@ typedef struct {                         /* type of structure for search result
   ESTRESDOC **survivors;                 /* pointers to surviving documents */
   int snum;                              /* number of surviving elements */
   CBMAP *hints;                          /* map object for hints */
+  time_t mdate;                          /* modification date */
 } ESTNODERES;
 
 
@@ -81,8 +82,8 @@ void est_noderes_delete(ESTNODERES *nres);
 /* Get a map object for hints of a node result object.
    `nres' specifies a node result object.
    The return value is a map object for hints.  Keys of the map are "VERSION", "NODE", "HIT",
-   "HINT#n", "DOCNUM", "WORDNUM", "TIME", "LINK#n", and "VIEW".  The life duration of the
-   returned object is synchronous with the one of the node result object. */
+   "HINT#n", "DOCNUM", "WORDNUM", "TIME", "TIME#n", "LINK#n", and "VIEW".  The life duration of
+   the returned object is synchronous with the one of the node result object. */
 CBMAP *est_noderes_hints(ESTNODERES *nres);
 
 
@@ -91,8 +92,7 @@ CBMAP *est_noderes_hints(ESTNODERES *nres);
    `num' specifies the number of documents to be shown.  If it is not more than 0, eclipse is
    undone.
    `limit' specifies the lower limit of similarity for documents to be eclipsed.  Similarity is
-   between 0.0 and 1.0.
-   This function does not work if the object was retrieved without ESTCONDETCH option. */
+   between 0.0 and 1.0. */
 void est_noderes_eclipse(ESTNODERES *nres, int num, double limit);
 
 
@@ -136,22 +136,20 @@ const char *est_resdoc_attr(ESTRESDOC *rdoc, const char *name);
 
 /* Get the snippet of a result document object.
    `rdoc' specifies a result document object.
-   The return value is the snippet of the result document object.  There are tab separated
-   values.  Each line is a string to be shown.  Though most lines have only one field, some
-   lines have two fields.  If the second field exists, the first field is to be shown with
+   The return value is a string of the snippet of the result document object.  There are tab
+   separated values.  Each line is a string to be shown.  Though most lines have only one field,
+   some lines have two fields.  If the second field exists, the first field is to be shown with
    highlighted, and the second field means its normalized form.  The life duration of the
    returned string is synchronous with the one of the result document object. */
 const char *est_resdoc_snippet(ESTRESDOC *rdoc);
 
 
-/* Get the keyword vector of a result document object.
+/* Get keywords of a result document object.
    `rdoc' specifies a result document object.
-   The return value is the keyword vector of the result document object.  There are tab separated
-   values.  Keywords and their scores come alternately.  The life duration of the returned string
-   is synchronous with the one of the result document object.
-   This function returns an empty string if the node result object was retrieved without
-   ESTCONDETCH option. */
-const char *est_resdoc_vector(ESTRESDOC *rdoc);
+   The return value is a string of serialized keywords of the result document object.  There are
+   tab separated values.  Keywords and their scores come alternately.  The life duration of the
+   returned string is synchronous with the one of the result document object. */
+const char *est_resdoc_keywords(ESTRESDOC *rdoc);
 
 
 /* Get an array of documents eclipsed by a result document object.
@@ -187,6 +185,9 @@ typedef struct {                         /* type of structure for a node object
   int dnum;                              /* number of documents */
   int wnum;                              /* number of words */
   double size;                           /* size of the database */
+  CBLIST *admins;                        /* list of names of administrators */
+  CBLIST *users;                         /* list of names of users */
+  CBLIST *links;                         /* list of expressions of linking nodes */
   int wwidth;                            /* whole width of a snippet */
   int hwidth;                            /* head width of a snippet */
   int awidth;                            /* around width of a snippet */
@@ -235,6 +236,18 @@ void est_node_set_auth(ESTNODE *node, const char *name, const char *passwd);
 int est_node_status(ESTNODE *node);
 
 
+/* Synchronize updating contents of the database of a node.
+   `node' specifies a node connection object.
+   The return value is true if success, else it is false. */
+int est_node_sync(ESTNODE *node);
+
+
+/* Optimize the database of a node.
+   `node' specifies a node connection object.
+   The return value is true if success, else it is false. */
+int est_node_optimize(ESTNODE *node);
+
+
 /* Add a document to a node.
    `node' specifies a node connection object.
    `doc' specifies a document object.  The document object should have the URI attribute.
@@ -359,7 +372,35 @@ int est_node_word_num(ESTNODE *node);
 double est_node_size(ESTNODE *node);
 
 
-/* Search documents corresponding a condition for a node.
+/* Get the usage ratio of the cache of a node.
+   `node' specifies a node connection object.
+   The return value is the usage ratio of the cache of the node.  On error, -1.0 is returned. */
+double est_node_cache_usage(ESTNODE *node);
+
+
+/* Get a list of names of administrators of a node.
+   `node' specifies a node connection object.
+   The return value is a list object of names of administrators.  On error, `NULL' is returned.
+   The life duration of the returned object is synchronous with the one of the node object. */
+const CBLIST *est_node_admins(ESTNODE *node);
+
+
+/* Get a list of names of users of a node.
+   `node' specifies a node connection object.
+   The return value is a list object of names of users.  On error, `NULL' is returned.  The life
+   duration of the returned object is synchronous with the one of the node object. */
+const CBLIST *est_node_users(ESTNODE *node);
+
+
+/* Get a list of expressions of links of a node.
+   `node' specifies a node connection object.
+   The return value is a list object of expressions of links.  Each element is a TSV string and
+   has three fields of the URL, the label, and the score.  On error, `NULL' is returned.  The
+   life duration of the returned object is synchronous with the one of the node object. */
+const CBLIST *est_node_links(ESTNODE *node);
+
+
+/* Search a node for documents corresponding a condition.
    `node' specifies a node connection object.
    `cond' specifies a condition object.
    `depth' specifies the depth of meta search.
@@ -503,6 +544,8 @@ void est_datum_printf(CBDATUM *datum, const char *format, ...);
    `pxhost' specifies the host name of a proxy.  If it is `NULL', it is not used.
    `pxport' specifies the port number of the proxy.
    `outsec' specifies timeout in seconds.  If it is negative, it is not used.
+   `limsize' specifies limitation of the entity body of response.  If it is negative, it is not
+   used.
    `auth' specifies an authority information in such form as "user:pass".  If it is `NULL', it is
    not used.
    `reqheads' specifies a list object of extension headers.  If it is `NULL' it is not used.
@@ -520,7 +563,7 @@ void est_datum_printf(CBDATUM *datum, const char *format, ...);
    The return value is true if success, else it is false.
    Headers of "Host", "Connection", "User-Agent", "Authorization", "Accept-Encoding", and
    "Content-Length" are sent implicitly. */
-int est_url_shuttle(const char *url, const char *pxhost, int pxport, int outsec,
+int est_url_shuttle(const char *url, const char *pxhost, int pxport, int outsec, int limsize,
                     const char *auth, const CBLIST *reqheads, const char *reqbody, int rbsiz,
                     int *rescodep, CBMAP *resheads, CBDATUM *resbody);
 
@@ -570,6 +613,12 @@ void est_noderes_add_doc(ESTNODERES *nres, CBMAP *attrs, char *snippet);
 int est_noderes_shift_doc(ESTNODERES *nres, CBMAP **attrp, char **snippetp);
 
 
+/* Get the modification date of a node result object.
+   `nres' specifies a node result object.
+   The return value is the modification date of a node result object. */
+time_t est_noderes_mdate(ESTNODERES *nres);
+
+
 
 #if defined(__cplusplus)                 /* export for C++ */
 }
