@@ -112,7 +112,7 @@ void est_doc_add_text(ESTDOC *doc, const char *text);
 void est_doc_add_hidden_text(ESTDOC *doc, const char *text);
 
 
-/* Attache keywords to a document object.
+/* Attach keywords to a document object.
    `doc' specifies a document object.
    `kwords' specifies a map object of keywords.  Keys of the map should be keywords of the
    document and values should be their scores in decimal string.  The map object is copied
@@ -122,7 +122,8 @@ void est_doc_set_keywords(ESTDOC *doc, CBMAP *kwords);
 
 /* Set the substitute score of a document object.
    `doc' specifies a document object.
-   `score' specifies the substitute score.  It should be zero or positive. */
+   `score' specifies the substitute score.  It it is negative, the substitute score setting is
+   nullified. */
 void est_doc_set_score(ESTDOC *doc, int score);
 
 
@@ -172,6 +173,12 @@ char *est_doc_cat_texts(ESTDOC *doc);
 CBMAP *est_doc_keywords(ESTDOC *doc);
 
 
+/* Get the substitute score of a document object.
+   `doc' specifies a document object.
+   The return value is the substitute score or -1 if it is not set. */
+int est_doc_score(ESTDOC *doc);
+
+
 /* Dump draft data of a document object.
    `doc' specifies a document object.
    The return value is draft data of the document object.  Because the region of the return value
@@ -264,6 +271,7 @@ typedef struct {                         /* type of structure for search conditi
   int opts;                              /* options for preservation */
   double ecllim;                         /* lower limit of similarity eclipse */
   CBMAP *shadows;                        /* map of eclipsed documents */
+  char *distinct;                        /* distinct attribute */
   int mask;                              /* mask for meta search */
 } ESTCOND;
 
@@ -353,6 +361,13 @@ void est_cond_set_auxiliary(ESTCOND *cond, int min);
 void est_cond_set_eclipse(ESTCOND *cond, double limit);
 
 
+/* Set the attribute distinction filter.
+   `cond' specifies a condition object.
+   `name' specifies the name of an attribute to be distinct.
+   If this filter is set, candidates which have same value of the attribute is omitted. */
+void est_cond_set_distinct(ESTCOND *cond, const char *name);
+
+
 /* Set the mask of targets of meta search.
    `cond' specifies a condition object.
    `mask' specifies a masking number.  1 means the first target, 2 means the second target, 4
@@ -368,6 +383,7 @@ void est_cond_set_mask(ESTCOND *cond, int mask);
 
 #define ESTIDXDMAX     256               /* max number of the inverted index */
 #define ESTIDXDSTD     16                /* standard number of the inverted index */
+#define ESTPDOCIDMIN   2000000001        /* minimum ID number of pseudo documents */
 
 typedef struct {                         /* type of structure for the inverted index */
   char *name;                            /* name of the database */
@@ -449,6 +465,8 @@ enum {                                   /* enumeration for open modes */
   ESTDBSMALL = 1 << 20,                  /* small tuning */
   ESTDBLARGE = 1 << 21,                  /* large tuning */
   ESTDBHUGE = 1 << 22,                   /* huge tuning */
+  ESTDBHUGE2 = 1 << 23,                  /* huge tuning second */
+  ESTDBHUGE3 = 1 << 24,                  /* huge tuning third */
   ESTDBSCVOID = 1 << 25,                 /* store scores as void */
   ESTDBSCINT = 1 << 26,                  /* store scores as integer */
   ESTDBSCASIS = 1 << 27                  /* refrain from adjustment of scores */
@@ -505,6 +523,8 @@ const char *est_err_msg(int ecode);
    `ESTDBSMALL', which means the index is tuned to register less than 50000 documents,
    `ESTDBLARGE', which means the index is tuned to register more than 300000 documents,
    `ESTDBHUGE', which means the index is tuned to register more than 1000000 documents,
+   `ESTDBHUGE2', which means the index is tuned to register more than 5000000 documents,
+   `ESTDBHUGE3', which means the index is tuned to register more than 10000000 documents,
    `ESTDBSCVOID', which means scores are stored as void, `ESTDBSCINT', which means scores are
    stored as 32-bit integer, `ESTDBSCASIS', which means scores are stored as-is and marked not
    to be tuned when search.
@@ -727,8 +747,8 @@ int est_db_add_pseudo_index(ESTDB *db, const char *path);
  *************************************************************************************************/
 
 
-#define _EST_VERSION   "1.4.2"
-#define _EST_LIBVER    827
+#define _EST_VERSION   "1.4.9"
+#define _EST_LIBVER    834
 #define _EST_PROTVER   "1.0"
 
 #define _EST_PROJURL   "http://hyperestraier.sourceforge.net/"
@@ -778,6 +798,18 @@ void est_break_text_perfng(const char *text, CBLIST *list, int norm, int tail);
 void est_break_text_chrcat(const char *text, CBLIST *list, int norm);
 
 
+/* Make a snippet of an arbitrary string.
+   `word' specifies a list object of words to be highlight.
+   `wwidth' specifies whole width of the result.
+   `hwidth' specifies width of strings picked up from the beginning of the text.
+   `awidth' specifies width of strings picked up around each highlighted word.
+   The return value is a snippet string of the string.  Because the region of the return value is
+   allocated with the `malloc' call, it should be released with the `free' call if it is no
+   longer in use. */
+char *est_str_make_snippet(const char *str, const CBLIST *words,
+                           int wwidth, int hwidth, int awidth);
+
+
 /* Convert the character encoding of a string.
    `ptr' specifies the pointer to a region.
    `size' specifies the size of the region.  If it is negative, the size is assigned with
@@ -959,11 +991,20 @@ void est_regex_delete(void *regex);
 int est_regex_match(const void *regex, const char *str);
 
 
+/* Check whether a regular expression matches a string.
+   `rstr' specifies a regular expression string.
+   `tstr' specifies a target string.
+   The return value is true if the regular expression string matchs the target string. */
+int est_regex_match_str(const char *rstr, const char *tstr);
+
+
 /* Replace each substring matching a regular expression string.
    `str' specifies a target string.
    `bef' specifies a string of regular expressions for substrings.
    `aft' specifies a string with which each substrings are replaced.  Each "&" in the string is
    replaced with the matched substring.  Each "\" in the string escapes the following character.
+   Special escapes "\1" through "\9" referring to the corresponding matching sub-expressions in
+   the regular expression string are supported.
    The return value is a new converted string.  Even if the regular expression is invalid, a copy
    of the original string is returned.  Because the region of the return value is allocated with
    the `malloc' call, it should be released with the `free' call if it is no longer in use. */
@@ -1055,6 +1096,13 @@ int est_cond_options(ESTCOND *cond);
 int est_cond_auxiliary(ESTCOND *cond);
 
 
+/* Get the attribute distinction filter.
+   `cond' specifies a condition object.
+   The return value is the name of the distinct attribute or `NULL' if it is not specified.  The
+   life duration of the returned string is synchronous with the one of the condition object. */
+const char *est_cond_distinct(ESTCOND *cond);
+
+
 /* Get the mask of targets of meta search.
    `cond' specifies a condition object.
    The return value is the mask of targets of meta search. */
@@ -1101,6 +1149,14 @@ void est_cond_set_expander(ESTCOND *cond, void (*func)(const char *, CBLIST *));
 void est_db_set_ecode(ESTDB *db, int ecode);
 
 
+/* Check whether an option is set.
+   `db' specifies a database object.
+   `option' specifies an option used when opening the database.
+   The return value is 1 if the option is set, 0 if the option is not set, or -1 if it is
+   unknown. */
+int est_db_check_option(ESTDB *db, int option);
+
+
 /* Get the inode number of a database.
    `db' specifies a database object.
    The return value is the inode number of the database. */
@@ -1329,7 +1385,7 @@ void est_db_refresh_rescc(ESTDB *db);
 void est_db_charge_rescc(ESTDB *db, int max);
 
 
-/* Get list of words in the result cache.
+/* Get a list of words in the result cache.
    `db' specifies a database object.
    The return value is a new list object of words in the result cache.  Because the object of the
    return value is opened with the function `cblistopen', it should be closed with the function
@@ -1337,6 +1393,20 @@ void est_db_charge_rescc(ESTDB *db, int max);
 CBLIST *est_db_list_rescc(ESTDB *db);
 
 
+/* Get the number of pseudo documents in a database.
+   `db' specifies a database object.
+   The return value is the number of pseudo documents in the database. */
+int est_db_pseudo_doc_num(ESTDB *db);
+
+
+/* Get a list of expressions of attribute indexes of a database.
+   `db' specifies a database object.
+   The return value is a new list object of expressions of attribute indexes.  Because the object
+   of the return value is opened with the function `cblistopen', it should be closed with the
+   function `cblistclose' if it is no longer in use. */
+CBLIST *est_db_attr_index_exprs(ESTDB *db);
+
+
 /* Interrupt long time processing.
    `db' specifies a database object. */
 void est_db_interrupt(ESTDB *db);
