@@ -63,7 +63,7 @@ static int runinit(int argc, char **argv);
 static int runcrawl(int argc, char **argv);
 static int rununittest(int argc, char **argv);
 static int runfetch(int argc, char **argv);
-static int procinit(const char *rootdir);
+static int procinit(const char *rootdir, int opts);
 static int proccrawl(const char *rootdir, int mode);
 static int procunittest(const char *rootdir);
 static int procfetch(const char *url);
@@ -115,7 +115,7 @@ static void usage(void){
   fprintf(stderr, "%s: command line interface of web crawler\n", g_progname);
   fprintf(stderr, "\n");
   fprintf(stderr, "usage:\n");
-  fprintf(stderr, "  %s init rootdir\n", g_progname);
+  fprintf(stderr, "  %s init [-xs|-xl|-xh] rootdir\n", g_progname);
   fprintf(stderr, "  %s crawl [-restart|-revisit|-revcont] rootdir\n", g_progname);
   fprintf(stderr, "  %s unittest rootdir\n", g_progname);
   fprintf(stderr, "  %s fetch [-proxy host port] [-tout num] [-il lang] url\n", g_progname);
@@ -347,11 +347,20 @@ static void enqueuelinks(WAVER *waver, const char *base, CBLIST *links, CBMAP *k
 /* parse arguments of the init command */
 static int runinit(int argc, char **argv){
   char *rootdir;
-  int i, rv;
+  int i, opts, rv;
   rootdir = NULL;
+  opts = 0;
   for(i = 2; i < argc; i++){
     if(!rootdir && argv[i][0] == '-'){
-      usage();
+      if(!strcmp(argv[i], "-xs")){
+        opts |= WI_SMALL;
+      } else if(!strcmp(argv[i], "-xl")){
+        opts |= WI_LARGE;
+      } else if(!strcmp(argv[i], "-xh")){
+        opts |= WI_HUGE;
+      } else {
+        usage();
+      }
     } else if(!rootdir){
       rootdir = argv[i];
     } else {
@@ -359,7 +368,7 @@ static int runinit(int argc, char **argv){
     }
   }
   if(!rootdir) usage();
-  rv = procinit(rootdir);
+  rv = procinit(rootdir, opts);
   return rv;
 }
 
@@ -447,8 +456,8 @@ static int runfetch(int argc, char **argv){
 
 
 /* perform the init command */
-static int procinit(const char *rootdir){
-  if(!waver_init(rootdir)){
+static int procinit(const char *rootdir, int opts){
+  if(!waver_init(rootdir, opts)){
     log_print(LL_ERROR, "initializing the root directory failed");
     return 1;
   }
@@ -654,7 +663,7 @@ static int procunittest(const char *rootdir){
   char uri[URIBUFSIZ], *vbuf;
   int i, err;
   log_print(LL_INFO, "initializing the waver handle");
-  if(!waver_init(rootdir)){
+  if(!waver_init(rootdir, 0)){
     log_print(LL_ERROR, "%s: initializing failed", rootdir);
     return FALSE;
   }
@@ -723,7 +732,7 @@ static int procfetch(const char *url){
   heads = cbmapopen();
   links = cblistopen();
   doc = est_doc_new();
-  if(!fetch_document(url, g_pxhost, g_pxport, g_timeout, -1, NULL, &code, raw, heads,
+  if(!fetch_document(url, g_pxhost, g_pxport, g_timeout, -1, NULL, NULL, &code, raw, heads,
                      links, NULL, doc, g_inputlang)){
     log_print(LL_WARN, "could not get: %d: %s", code, url);
     est_doc_delete(doc);
@@ -830,7 +839,8 @@ static void seedurldocs(WAVER *waver, const char *url, int depth, double bias, C
   doc = est_doc_new();
   log_print(LL_INFO, "fetching: %d: %s", depth, url);
   fetch_document(url, waver->pxhost, waver->pxport, waver->timeout * 2, 0,
-                 waver->mtrules, &code, NULL, NULL, links, waver->unrules, doc, waver->language);
+                 waver->urlrules, waver->mtrules, &code, NULL, NULL, links,
+                 waver->unrules, doc, waver->language);
   if(code == 200 && !est_doc_is_empty(doc)){
     log_print(LL_INFO, "seeding: %.3f: %s", bias, url);
     kwords = est_morph_etch_doc(doc, waver->seedkeynum);
@@ -951,7 +961,8 @@ static void *geturldoc(void *args){
   doc = est_doc_new();
   log_print(LL_INFO, "[%d]: fetching: %d: %s", thid, myargs->depth, url);
   fetch_document(url, waver->pxhost, waver->pxport, waver->timeout, myargs->mdate,
-                 waver->mtrules, &code, raw, heads, links, waver->unrules, doc, waver->language);
+                 waver->urlrules, waver->mtrules, &code, raw, heads, links,
+                 waver->unrules, doc, waver->language);
   if(g_sigterm){
     log_print(LL_WARN, "[%d]: terminated: %s", thid, url);
     if(pthread_mutex_lock(&g_mutex) == 0){
