@@ -32,6 +32,7 @@ public class Cmd {
   /* class constants */
   private static final String PROTVER = "1.0";
   private static final int SEARCHMAX = 10;
+  private static final int SEARCHAUX = 32;
   private static final int SNIPWWIDTH = 480;
   private static final int SNIPHWIDTH = 96;
   private static final int SNIPAWIDTH = 96;
@@ -97,7 +98,8 @@ public class Cmd {
     stderr.println("  optimize [-onp] [-ond] db");
     stderr.println("  merge [-cl] db target");
     stderr.println("  search [-vu|-va|-vf|-vs] [-gs|-gf|-ga] [-cd] [-ni] [-sf|-sfr|-sfu|-sfi]" +
-                   " [-attr expr] [-ord expr] [-max num] [-sk num] db [phrase]");
+                   " [-attr expr] [-ord expr] [-max num] [-sk num] [-aux num] [-dis name]" +
+                   " db [phrase]");
     stderr.println("");
     System.exit(1);
   }
@@ -294,6 +296,8 @@ public class Cmd {
     int max = SEARCHMAX;
     int skip = 0;
     int opts = 0;
+    int aux = SEARCHAUX;
+    String dis = null;
     boolean cd = false;
     int view = VM_ID;
     for(int i = 1; i < args.length; i++){
@@ -337,6 +341,12 @@ public class Cmd {
         } else if(arg.equals("-sk")){
           if(++i >= args.length) usage();
           skip = Integer.parseInt(args[i]);
+        } else if(arg.equals("-aux")){
+          if(++i >= args.length) usage();
+          aux = Integer.parseInt(args[i]);
+        } else if(arg.equals("-dis")){
+          if(++i >= args.length) usage();
+          dis = args[i];
         } else {
           usage();
         }
@@ -349,7 +359,7 @@ public class Cmd {
       }
     }
     if(dbname == null) usage();
-    return procsearch(dbname, phrase, attrs, ord, max, skip, opts, cd, view);
+    return procsearch(dbname, phrase, attrs, ord, max, skip, opts, aux, dis, cd, view);
   }
   /* perform the put command */
   private static int procput(String dbname, String file, int opts){
@@ -584,7 +594,8 @@ public class Cmd {
   }
   /* perform the search command */
   private static int procsearch(String dbname, String phrase, List attrs, String ord,
-                                int max, int skip, int opts, boolean cd, int view){
+                                int max, int skip, int opts, int aux, String dis,
+                                boolean cd, int view){
     Database db = new Database();
     if(!db.open(dbname, Database.DBREADER)){
       printerror(dbname + ": " + db.err_msg(db.error()));
@@ -600,6 +611,8 @@ public class Cmd {
     if(max >= 0) cond.set_max(max);
     if(skip >= 0) cond.set_skip(skip);
     cond.set_options(opts);
+    cond.set_auxiliary(aux);
+    if(dis != null) cond.set_distinct(dis);
     long stime = new GregorianCalendar().getTimeInMillis();
     Result res = db.search(cond);
     long etime = new GregorianCalendar().getTimeInMillis();
