@@ -75,6 +75,10 @@ public class Database {
   public static final int DBLARGE = 1 << 21;
   /** open mode: huge tuning */
   public static final int DBHUGE = 1 << 22;
+  /** open mode: huge tuning second */
+  public static final int DBHUGE2 = 1 << 23;
+  /** open mode: huge tuning third */
+  public static final int DBHUGE3 = 1 << 24;
   /** open mode: store scores as void */
   public static final int DBSCVOID = 1 << 25;
   /** open mode: store scores as integer */
@@ -173,6 +177,8 @@ public class Database {
    * analysis, `Database.DBSMALL', which means the index is tuned to register less than 50000
    * documents, `Database.DBLARGE', which means the index is tuned to register more than 300000
    * documents, `Database.DBHUGE', which means the index is tuned to register more than 1000000
+   * documents, `Database.DBHUGE2', which means the index is tuned to register more than 5000000
+   * documents, `Database.DBHUGE3', which means the index is tuned to register more than 10000000
    * documents, `Database.DBSCVOID', which means scores are stored as void, `Database.DBSCINT',
    * which means scores are stored as 32-bit integer, `Database.DBSCASIS', which means scores
    * are stored as-is and marked not to be tuned when search.
