@@ -169,6 +169,30 @@ Java_estraier_Condition_set_1eclipse(JNIEnv *env, jobject obj, jdouble limit){
 }
 
 
+/* set_distinct */
+JNIEXPORT void JNICALL
+Java_estraier_Condition_set_1distinct(JNIEnv *env, jobject obj, jstring name){
+  jclass cls;
+  jfieldID fid;
+  jlong coreptr;
+  jboolean icname;
+  const char *tname;
+  cls = (*env)->GetObjectClass(env, obj);
+  fid = (*env)->GetFieldID(env, cls, "coreptr", "J");
+  coreptr = (*env)->GetLongField(env, obj, fid);
+  if(!name || !isinstanceof(env, name, CLSSTRING)){
+    throwillarg(env);
+    return;
+  }
+  if(!(tname = (*env)->GetStringUTFChars(env, name, &icname))){
+    throwoutmem(env);
+    return;
+  }
+  est_cond_set_distinct((ESTCOND *)(PTRNUM)coreptr, tname);
+  if(icname == JNI_TRUE) (*env)->ReleaseStringUTFChars(env, name, tname);
+}
+
+
 /* set_mask */
 JNIEXPORT void JNICALL
 Java_estraier_Condition_set_1mask(JNIEnv *env, jobject obj, jint mask){
