@@ -670,6 +670,40 @@ Java_estraier_Database_set_1cache_1size(JNIEnv *env, jobject obj,
 }
 
 
+/* add_pseudo_index */
+JNIEXPORT jboolean JNICALL
+Java_estraier_Database_add_1pseudo_1index(JNIEnv *env, jobject obj, jstring path){
+  jclass cls;
+  jfieldID fid;
+  jlong coreptr;
+  jboolean icpath;
+  const char *tpath;
+  int err;
+  cls = (*env)->GetObjectClass(env, obj);
+  fid = (*env)->GetFieldID(env, cls, "coreptr", "J");
+  coreptr = (*env)->GetLongField(env, obj, fid);
+  if(coreptr == 0){
+    throwillarg(env);
+    return FALSE;
+  }
+  if(!path || !isinstanceof(env, path, CLSSTRING)){
+    throwillarg(env);
+    return FALSE;
+  }
+  if(!(tpath = (*env)->GetStringUTFChars(env, path, &icpath))){
+    throwoutmem(env);
+    return FALSE;
+  }
+  err = FALSE;
+  if(!est_mtdb_add_pseudo_index((ESTMTDB *)(PTRNUM)coreptr, tpath)){
+    setecode(env, obj, est_mtdb_error((ESTMTDB *)(PTRNUM)coreptr));
+    err = TRUE;
+  }
+  if(icpath == JNI_TRUE) (*env)->ReleaseStringUTFChars(env, path, tpath);
+  return err ? FALSE : TRUE;
+}
+
+
 /* set_wildmax */
 JNIEXPORT void JNICALL
 Java_estraier_Database_set_1wildmax(JNIEnv *env, jobject obj, jint num){
