@@ -44,6 +44,7 @@ public class Call {
   //----------------------------------------------------------------
   /* class constants */
   private static final int SEARCHMAX = 10;
+  private static final int SEARCHAUX = 32;
   private static final PrintStream stdout = System.out;
   private static final PrintStream stderr = System.err;
   /* class variables */
@@ -124,7 +125,7 @@ public class Call {
                    " nurl");
     stderr.println("  search [-proxy host:port] [-tout num] [-auth user pass]" +
                    " [-sf|-sfr|-sfu|-sfi] [-attr expr] [-ord expr] [-max num] [-sk num]" +
-                   " [-dpt num] [-mask num] nurl [phrase]");
+                   " [-aux num] [-dis name] [-dpt num] [-mask num] nurl [phrase]");
     stderr.println("  setuser [-proxy host:port] [-tout num] [-auth user pass]" +
                    " nurl name mode");
     stderr.println("  setlink [-proxy host:port] [-tout num] [-auth user pass]" +
@@ -452,6 +453,8 @@ public class Call {
     int max = SEARCHMAX;
     int skip = 0;
     int opts = 0;
+    int aux = SEARCHAUX;
+    String dis = null;
     int depth = 0;
     int mask = 0;
     for(int i = 1; i < args.length; i++){
@@ -490,6 +493,12 @@ public class Call {
         } else if(arg.equals("-sk")){
           if(++i >= args.length) usage();
           skip = Integer.parseInt(args[i]);
+        } else if(arg.equals("-aux")){
+          if(++i >= args.length) usage();
+          aux = Integer.parseInt(args[i]);
+        } else if(arg.equals("-dis")){
+          if(++i >= args.length) usage();
+          dis = args[i];
         } else if(arg.equals("-dpt")){
           if(++i >= args.length) usage();
           depth = Integer.parseInt(args[i]);
@@ -508,7 +517,7 @@ public class Call {
       }
     }
     if(nurl == null) usage();
-    return procsearch(nurl, phrase, attrs, ord, max, skip, opts, depth, mask);
+    return procsearch(nurl, phrase, attrs, ord, max, skip, opts, aux, dis, depth, mask);
   }
   /* parse arguments of the setuser command */
   private static int runsetuser(String[] args){
@@ -803,7 +812,8 @@ public class Call {
   }
   /* perform the search command */
   private static int procsearch(String nurl, String phrase, List attrs, String ord,
-                                int max, int skip, int opts, int depth, int mask){
+                                int max, int skip, int opts, int aux, String dis,
+                                int depth, int mask){
     Node node = new Node();
     node.set_url(nurl);
     if(pxhost != null) node.set_proxy(pxhost, pxport);
@@ -819,6 +829,8 @@ public class Call {
     if(max >= 0) cond.set_max(max);
     if(skip >= 0) cond.set_skip(skip);
     cond.set_options(opts);
+    cond.set_auxiliary(aux);
+    if(dis != null) cond.set_distinct(dis);
     cond.set_mask(mask);
     NodeResult nres = node.search(cond, depth);
     Calendar cal = new GregorianCalendar();
