@@ -47,6 +47,7 @@ public class Document {
   private List dtexts;
   private List htexts;
   private Map kwords;
+  private int score;
   //----------------------------------------------------------------
   // constructors
   //----------------------------------------------------------------
@@ -59,6 +60,7 @@ public class Document {
     dtexts = new ArrayList(31);
     htexts = new ArrayList(31);
     kwords = null;
+    score = -1;
   }
   /**
    * Create a document object made from draft data.
@@ -72,7 +74,7 @@ public class Document {
       String line = lines[lnum++].trim();
       if(line.length() < 1) break;
       if(line.startsWith("%")){
-        if(line.startsWith("%VECTOR")){
+        if(line.startsWith("%VECTOR\t")){
           if(kwords == null) kwords = new HashMap(32);
           String[] fields = Utility.split_fields(line);
           for(int i = 1; i < fields.length - 1; i += 2){
@@ -80,6 +82,9 @@ public class Document {
                fields[i].charAt(0) <= ' ') continue;
             kwords.put(fields[i], fields[i+1]);
           }
+        } else if(line.startsWith("%SCORE\t")){
+          String[] fields = Utility.split_fields(line);
+          score = Integer.parseInt(fields[1]);
         }
         continue;
       }
@@ -129,13 +134,21 @@ public class Document {
     if(text.length() > 0) htexts.add(text);
   }
   /**
-   * Attache keywords.
+   * Attach keywords.
    * @param kwords a map object of keywords.  Keys of the map should be keywords of the document
    * and values should be their scores in decimal string.
    */
   public void set_keywords(Map kwords){
     this.kwords = kwords;
   }
+  /**
+   * Set the substitute score.
+   * @param score the substitute score.  It it is negative, the substitute score setting is
+   * nullified.
+   */
+  private void set_score(int score){
+    this.score = score;
+  }
   /**
    * Get the ID number.
    * @return the ID number.  If this object has never been registered, -1 is returned.
@@ -192,6 +205,14 @@ public class Document {
   public Map keywords(){
     return kwords;
   }
+  /**
+   * Get the substitute score.
+   * @return the substitute score or -1 if it is not set.
+   */
+  public int score(){
+    if(score < 0) return -1;
+    return score;
+  }
   /**
    * Dump draft data.
    * @return draft data.
@@ -219,6 +240,7 @@ public class Document {
       }
       sb.append("\n");
     }
+    if(score >= 0) sb.append("%SCORE\t" + score + "\n");
     sb.append("\n");
     Iterator dtit = dtexts.iterator();
     while(dtit.hasNext()){
