@@ -67,7 +67,7 @@ public class Node {
         sb.append(URLEncoder.encode((String)it.next(), "UTF-8"));
       }
       String order = cond.order();
-      if(order != null){
+      if(order != null && order.length() > 0){
         sb.append("&order=");
         sb.append(URLEncoder.encode(order, "UTF-8"));
       }
@@ -87,6 +87,11 @@ public class Node {
       int auxiliary = cond.auxiliary();
       sb.append("&auxiliary=");
       sb.append(auxiliary);
+      String distinct = cond.distinct();
+      if(distinct != null && distinct.length() > 0){
+        sb.append("&distinct=");
+        sb.append(URLEncoder.encode(distinct, "UTF-8"));
+      }
       if(depth > 0){
         sb.append("&depth=");
         sb.append(depth);
