@@ -195,7 +195,7 @@ int master_init(const char *rootdir){
     fprintf(ofp, "# whether to rate URI for scoring (0:no, 1:yes)\n");
     fprintf(ofp, "rateuri: 1\n");
     fprintf(ofp, "\n");
-    fprintf(ofp, "# merge method (1:score, 2:score and rank, 3:rank)\n");
+    fprintf(ofp, "# merge method of meta search (1:score, 2:score and rank, 3:rank)\n");
     fprintf(ofp, "mergemethod: 2\n");
     fprintf(ofp, "\n");
     fprintf(ofp, "# host name of the proxy\n");
@@ -1012,11 +1012,12 @@ void resmap_put(RESMAP *resmap, int score, ESTDOC *doc, CBMAP *attrs, char *body
 
 
 /* Get a list object of result objects in a result map objects. */
-RESDOC **resmap_list(RESMAP *resmap, int *nump, const char *order){
-  RESDOC **resdocs;
+RESDOC **resmap_list(RESMAP *resmap, int *nump, const char *order, const char *distinct){
+  RESDOC **resdocs, *resdoc;
+  CBMAP *umap;
   const char *kbuf, *vbuf, *otype, *rp;
   char *oname, *wp;
-  int i, onlen;
+  int i, onlen, dnlen, nnum;
   time_t tval;
   assert(resmap && nump);
   if(pthread_mutex_lock(&(resmap->mutex)) != 0){
@@ -1073,6 +1074,24 @@ RESDOC **resmap_list(RESMAP *resmap, int *nump, const char *order){
   } else {
     qsort(resdocs, *nump, sizeof(RESDOC *), resdoc_compare_by_score);
   }
+  if(distinct){
+    dnlen = strlen(distinct);
+    umap = cbmapopenex(*nump + 1);
+    nnum = 0;
+    for(i = 0; i < *nump; i++){
+      resdoc = resdocs[i];
+      if(resdoc->doc){
+        vbuf = est_doc_attr(resdoc->doc, distinct);
+        printf("*");
+      } else {
+        vbuf = cbmapget(resdoc->attrs, distinct, dnlen, NULL);
+      }
+      if(!vbuf) vbuf = "";
+      if(cbmapput(umap, vbuf, -1, "", 0, FALSE)) resdocs[nnum++] = resdoc;
+    }
+    *nump = nnum;
+    cbmapclose(umap);
+  }
   pthread_mutex_unlock(&(resmap->mutex));
   return resdocs;
 }
