@@ -21,6 +21,7 @@
 #include "estmtdb.h"
 #include "estnode.h"
 #include "myconf.h"
+#include "mymorph.h"
 
 
 
@@ -445,10 +446,11 @@ void resmap_put(RESMAP *resmap, int score, ESTDOC *doc, CBMAP *attrs, char *body
    assigned.
    `order' specifies an expression for the order.  If it is `NULL', the order is by score
    descending.
+   `distinct' specifies the name of the distinct attribute.  If it is `NULL', no filter is used.
    The return value is an array whose elements are result document objects.  Because the region
    of the return value is allocated with the `malloc' call, it should be released with the `free'
    call if it is no longer in use. */
-RESDOC **resmap_list(RESMAP *resmap, int *nump, const char *order);
+RESDOC **resmap_list(RESMAP *resmap, int *nump, const char *order, const char *distinct);
 
 
 /* Be a daemon process.
