@@ -29,6 +29,7 @@ use constant {
     FALSE => 0,
     PROTVER => "1.0",
     SEARCHMAX => 10,
+    SEARCHAUX => 32,
     SNIPWWIDTH => 480,
     SNIPHWIDTH => 96,
     SNIPAWIDTH => 96,
@@ -84,7 +85,8 @@ sub usage {
     printf(STDERR "  %s optimize [-onp] [-ond] db\n", $0);
     printf(STDERR "  %s merge [-cl] db target\n", $0);
     printf(STDERR "  %s search [-vu|-va|-vf|-vs] [-gs|-gf|-ga] [-cd] [-ni] [-sf|-sfr|-sfu|-sfi]" .
-           " [-attr expr] [-ord expr] [-max num] [-sk num] db [phrase]\n", $0);
+           " [-attr expr] [-ord expr] [-max num] [-sk num] [-aux num] [-dis name]" .
+           " db [phrase]\n", $0);
     printf(STDERR "\n");
     exit(1);
 }
@@ -302,6 +304,8 @@ sub runsearch {
     my $max = SEARCHMAX;
     my $skip = 0;
     my $opts = 0;
+    my $aux = SEARCHAUX;
+    my $dis = undef;
     my $cd = FALSE;
     my $view = VM_ID;
     my $i;
@@ -345,6 +349,12 @@ sub runsearch {
             } elsif($ARGV[$i] eq "-sk"){
                 usage() if(++$i >= scalar(@ARGV));
                 $skip = Estraier::atoi($ARGV[$i]);
+            } elsif($ARGV[$i] eq "-aux"){
+                usage() if(++$i >= scalar(@ARGV));
+                $aux = Estraier::atoi($ARGV[$i]);
+            } elsif($ARGV[$i] eq "-dis"){
+                usage() if(++$i >= scalar(@ARGV));
+                $dis = $ARGV[$i];
             } else {
                 usage();
             }
@@ -357,7 +367,7 @@ sub runsearch {
         }
     }
     usage() if(!defined($dbname));
-    procsearch($dbname, $phrase, \@attrs, $ord, $max, $skip, $opts, $cd, $view);
+    procsearch($dbname, $phrase, \@attrs, $ord, $max, $skip, $opts, $aux, $dis, $cd, $view);
 }
 
 
@@ -608,6 +618,8 @@ sub procsearch {
     my $max = shift;
     my $skip = shift;
     my $opts = shift;
+    my $aux = shift;
+    my $dis = shift;
     my $cd = shift;
     my $view = shift;
     my $db = new Database();
@@ -624,6 +636,8 @@ sub procsearch {
     $cond->set_max($max) if($max >= 0);
     $cond->set_skip($skip) if($skip >= 0);
     $cond->set_options($opts);
+    $cond->set_auxiliary($aux);
+    $cond->set_distinct($dis) if(defined($dis));
     my ($sec, $usec) = gettimeofday();
     my $stime = $sec + $usec / 1000000.0;
     my $res = $db->search($cond);
