@@ -215,6 +215,16 @@ sub keywords {
 }
 
 
+sub score {
+    my $self = shift;
+    if(scalar(@_) != 0){
+        croak(__FILE__ . ": invalid parameter") if($Estraier::DEBUG);
+        return undef;
+    }
+    return Estraier::doc_score($$self[0]);
+}
+
+
 sub dump_draft {
     my $self = shift;
     if(scalar(@_) != 0){
@@ -380,6 +390,18 @@ sub set_eclipse {
 }
 
 
+sub set_distinct {
+    my $self = shift;
+    my $name = shift;
+    if(scalar(@_) != 0 || !defined($name)){
+        croak(__FILE__ . ": invalid parameter") if($Estraier::DEBUG);
+        return undef;
+    }
+    Estraier::cond_set_distinct($$self[0], $name);
+    return undef;
+}
+
+
 
 package Result;
 use Carp;
@@ -519,6 +541,8 @@ use constant {
     DBSMALL => 1 << 20,
     DBLARGE => 1 << 21,
     DBHUGE => 1 << 22,
+    DBHUGE2 => 1 << 23,
+    DBHUGE3 => 1 << 24,
     DBSCVOID => 1 << 25,
     DBSCINT => 1 << 26,
     DBSCASIS => 1 << 27,
