@@ -47,7 +47,7 @@ module Estraier
     def add_hidden_text(text)
       # native code ...
     end
-    # Attache keywords.
+    # Attach keywords.
     # `kwords' specifies a hash object of keywords.  Keys of the hash should be keywords of the
     # document and values should be their scores in decimal string.
     # The return value is always `nil'.
@@ -55,7 +55,8 @@ module Estraier
       # native code ...
     end
     # Set the substitute score.
-    # `score' specifies the substitute score.  It should be zero or positive.
+    # `score' specifies the substitute score.  It it is negative, the substitute score setting is
+    # nullified.
     # The return value is always `nil'.
     def set_score(score)
       # native code ...
@@ -93,6 +94,11 @@ module Estraier
     def keywords()
       # native code ...
     end
+    # Get the substitute score.
+    # The return value is the substitute score or -1 if it is not set.
+    def score()
+      # native code ...
+    end
     # Dump draft data of a document object.
     # The return value is draft data.
     def dump_draft()
@@ -220,6 +226,11 @@ module Estraier
     def set_eclipse(limit)
       # native code ...
     end
+    # Set the attribute distinction filter.
+    # `name' specifies the name of an attribute to be distinct.
+    # The return value is always `nil'.
+    def set_distinct(name)
+    end
     # Set the mask of targets of meta search.
     # `mask' specifies a masking number.  1 means the first target, 2 means the second target, 4
     # means the third target, and power values of 2 and their summation compose the mask.
@@ -344,6 +355,10 @@ module Estraier
     DBLARGE = 1 << 21
     # open mode: huge tuning
     DBHUGE = 1 << 22
+    # open mode: huge tuning second
+    DBHUGE2 = 1 << 23
+    # open mode: huge tuning third
+    DBHUGE3 = 1 << 24
     # open mode: store scores as void
     DBSCVOID = 1 << 25
     # open mode: store scores as integer
@@ -410,7 +425,9 @@ module Estraier
     # character category analysis is performed instead of N-gram analysis, `Database::DBSMALL',
     # which means the index is tuned to register less than 50000 documents, `Database::DBLARGE',
     # which means the index is tuned to register more than 300000 documents, `Database::DBHUGE',
-    # which means the index is tuned to register more than 1000000 documents,
+    # which means the index is tuned to register more than 1000000 documents, `Database::DBHUGE2',
+    # which means the index is tuned to register more than 5000000 documents, `Database::DBHUGE3',
+    # which means the index is tuned to register more than 10000000 documents,
     # `Database::DBSCVOID', which means scores are stored as void, `Database::DBSCINT', which
     # means scores are stored as 32-bit integer, `Database::DBSCASIS', which means scores are
     # stored as-is and marked not to be tuned when search.
