@@ -51,6 +51,7 @@ static VALUE doc_attr(VALUE vself, VALUE vname);
 static VALUE doc_texts(VALUE vself);
 static VALUE doc_cat_texts(VALUE vself);
 static VALUE doc_keywords(VALUE vself);
+static VALUE doc_score(VALUE vself);
 static VALUE doc_dump_draft(VALUE vself);
 static VALUE doc_make_snippet(VALUE vself, VALUE vwords,
                               VALUE vwwidth, VALUE vhwidth, VALUE vawidth);
@@ -64,6 +65,7 @@ static VALUE cond_set_skip(VALUE vself, VALUE vskip);
 static VALUE cond_set_options(VALUE vself, VALUE voptions);
 static VALUE cond_set_auxiliary(VALUE vself, VALUE vmin);
 static VALUE cond_set_eclipse(VALUE vself, VALUE vlimit);
+static VALUE cond_set_distinct(VALUE vself, VALUE vname);
 static VALUE cond_set_mask(VALUE vself, VALUE vmask);
 static void res_init(void);
 static VALUE res_initialize(VALUE vself);
@@ -164,6 +166,7 @@ static void doc_init(void){
   rb_define_method(cls_doc, "texts", doc_texts, 0);
   rb_define_method(cls_doc, "cat_texts", doc_cat_texts, 0);
   rb_define_method(cls_doc, "keywords", doc_keywords, 0);
+  rb_define_method(cls_doc, "score", doc_score, 0);
   rb_define_method(cls_doc, "dump_draft", doc_dump_draft, 0);
   rb_define_method(cls_doc, "make_snippet", doc_make_snippet, 4);
 }
@@ -242,12 +245,9 @@ static VALUE doc_set_keywords(VALUE vself, VALUE vkwords){
 static VALUE doc_set_score(VALUE vself, VALUE vscore){
   VALUE vdoc;
   ESTDOC *doc;
-  int score;
   vdoc = rb_iv_get(vself, VNDATA);
   Data_Get_Struct(vdoc, ESTDOC, doc);
-  score = NUM2INT(vscore);
-  if(score < 0) rb_raise(rb_eArgError, "invalid argument");
-  est_doc_set_score(doc, score);
+  est_doc_set_score(doc, NUM2INT(vscore));
   return Qnil;
 }
 
@@ -319,6 +319,15 @@ static VALUE doc_keywords(VALUE vself){
 }
 
 
+static VALUE doc_score(VALUE vself){
+  VALUE vdoc;
+  ESTDOC *doc;
+  vdoc = rb_iv_get(vself, VNDATA);
+  Data_Get_Struct(vdoc, ESTDOC, doc);
+  return INT2NUM(est_doc_score(doc));
+}
+
+
 static VALUE doc_dump_draft(VALUE vself){
   VALUE vdoc, vdraft;
   ESTDOC *doc;
@@ -381,6 +390,7 @@ static void cond_init(void){
   rb_define_method(cls_cond, "set_options", cond_set_options, 1);
   rb_define_method(cls_cond, "set_auxiliary", cond_set_auxiliary, 1);
   rb_define_method(cls_cond, "set_eclipse", cond_set_eclipse, 1);
+  rb_define_method(cls_cond, "set_distinct", cond_set_distinct, 1);
   rb_define_method(cls_cond, "set_mask", cond_set_mask, 1);
 }
 
@@ -485,6 +495,17 @@ static VALUE cond_set_eclipse(VALUE vself, VALUE vlimit){
 }
 
 
+static VALUE cond_set_distinct(VALUE vself, VALUE vname){
+  VALUE vcond;
+  ESTCOND *cond;
+  vcond = rb_iv_get(vself, VNDATA);
+  Data_Get_Struct(vcond, ESTCOND, cond);
+  Check_Type(vname, T_STRING);
+  est_cond_set_distinct(cond, StringValuePtr(vname));
+  return Qnil;
+}
+
+
 static VALUE cond_set_mask(VALUE vself, VALUE vmask){
   VALUE vcond;
   ESTCOND *cond;
@@ -661,6 +682,8 @@ static void db_init(void){
   rb_define_const(cls_db, "DBSMALL", INT2NUM(ESTDBSMALL));
   rb_define_const(cls_db, "DBLARGE", INT2NUM(ESTDBLARGE));
   rb_define_const(cls_db, "DBHUGE", INT2NUM(ESTDBHUGE));
+  rb_define_const(cls_db, "DBHUGE2", INT2NUM(ESTDBHUGE2));
+  rb_define_const(cls_db, "DBHUGE3", INT2NUM(ESTDBHUGE3));
   rb_define_const(cls_db, "DBSCVOID", INT2NUM(ESTDBSCVOID));
   rb_define_const(cls_db, "DBSCINT", INT2NUM(ESTDBSCINT));
   rb_define_const(cls_db, "DBSCASIS", INT2NUM(ESTDBSCASIS));
