@@ -34,6 +34,7 @@ include EstraierPure
 
 # global constants
 SEARCHMAX = 10
+SEARCHAUX = 10
 
 # global variables
 $pxhost = nil
@@ -98,7 +99,7 @@ def usage()
   STDERR.printf("  %s optimize [-proxy host port] [-tout num] [-auth user pass] nurl\n", $0)
   STDERR.printf("  %s search [-proxy host port] [-tout num] [-auth user pass]" \
                 " [-sf|-sfr|-sfu|-sfi] [-attr expr] [-ord expr] [-max num] [-sk num]" \
-                " [-dpt num] [-mask num] nurl [phrase]\n", $0)
+                " [-aux num] [-dis name] [-dpt num] [-mask num] nurl [phrase]\n", $0)
   STDERR.printf("  %s setuser [-proxy host port] [-tout num] [-auth user pass]" \
                 " nurl name mode\n", $0)
   STDERR.printf("  %s setlink [-proxy host port] [-tout num] [-auth user pass]" \
@@ -457,6 +458,8 @@ def runsearch(args)
   max = SEARCHMAX
   skip = 0
   opts = 0
+  aux = SEARCHAUX
+  dis = nil
   depth = 0
   mask = 0
   i = 1
@@ -495,6 +498,12 @@ def runsearch(args)
       elsif args[i] == "-sk"
         usage if (i += 1) >= args.length
         skip = args[i].to_i
+      elsif args[i] == "-aux"
+        usage if (i += 1) >= args.length
+        aux = args[i].to_i
+      elsif args[i] == "-dis"
+        usage if (i += 1) >= args.length
+        dis = args[i]
       elsif args[i] == "-dpt"
         usage if (i += 1) >= args.length
         depth = args[i].to_i
@@ -514,7 +523,7 @@ def runsearch(args)
     i += 1
   end
   usage if !nurl || depth < 0
-  procsearch(nurl, phrase, attrs, ord, max, skip, opts, depth, mask)
+  procsearch(nurl, phrase, attrs, ord, max, skip, opts, aux, dis, depth, mask)
 end
 
 
@@ -790,7 +799,7 @@ end
 
 
 # perform the search command
-def procsearch(nurl, phrase, attrs, ord, max, skip, opts, depth, mask)
+def procsearch(nurl, phrase, attrs, ord, max, skip, opts, aux, dis, depth, mask)
   node = Node::new
   node.set_url(nurl)
   node.set_proxy($pxhost, $pxport) if $pxhost
@@ -805,6 +814,8 @@ def procsearch(nurl, phrase, attrs, ord, max, skip, opts, depth, mask)
   cond.set_max(max) if max >= 0
   cond.set_skip(skip) if skip >= 0
   cond.set_options(opts)
+  cond.set_auxiliary(aux)
+  cond.set_distinct(dis) if dis
   cond.set_mask(mask)
   nres = node.search(cond, depth)
   if !nres
