@@ -29,6 +29,7 @@
 
 
 require "uri"
+require "cgi"
 require "socket"
 require "stringio"
 
@@ -79,7 +80,7 @@ module EstraierPure
       @htexts.push(text) if text.length
       nil
     end
-    # Attache keywords.
+    # Attach keywords.
     # `kwords' specifies a map object of keywords.  Keys of the map should be keywords of the
     # document and values should be their scores in decimal string.
     # The return value is always `nil'.
@@ -88,6 +89,15 @@ module EstraierPure
       @kwords = kwords
       nil
     end
+    # Set the substitute score.
+    # `score' specifies the substitute score.  It it is negative, the substitute score setting is
+    # nullified.
+    # The return value is always `nil'.
+    def set_score(score)
+      Utility::check_types({ score=>Integer }) if $DEBUG
+      @score = score
+      nil
+    end
     # Get the ID number.
     # The return value is the ID number of the document object.  If the object has never been
     # registered, -1 is returned.
@@ -136,6 +146,7 @@ module EstraierPure
         end
         buf.printf("\n")
       end
+      buf.printf("%%SCORE\t%d\n", @score) if @score >= 0
       buf.printf("\n")
       for i in 0...@dtexts.length
         buf.printf("%s\n", @dtexts[i])
@@ -151,6 +162,12 @@ module EstraierPure
     def keywords()
       @kwords
     end
+    # Get the substitute score.
+    # The return value is the substitute score or -1 if it is not set.
+    def score()
+      return -1 if(@score < 0)
+      @score
+    end
     #--------------------------------
     # private methods
     #--------------------------------
@@ -164,6 +181,7 @@ module EstraierPure
       @dtexts = []
       @htexts = []
       @kwords = nil
+      @score = -1
       if draft.length
         lines = draft.split(/\n/, -1)
         num = 0
@@ -180,6 +198,9 @@ module EstraierPure
                 @kwords[fields[i]] = fields[i+1]
                 i += 2
               end
+            elsif line =~ /^%SCORE\t/
+              fields = line.split(/\t/)
+              @score = fields[1].to_i;
             end
             next
           end
@@ -193,13 +214,13 @@ module EstraierPure
         end
         while num < lines.length
           line = lines[num]
-          next unless line.length
+          num += 1
+          next unless line.length > 0
           if line[0] == 0x9
             @htexts.push(line[1...line.length]) if line.length > 1
           else
             @dtexts.push(line)
           end
-          num += 1
         end
       end
     end
@@ -305,6 +326,16 @@ module EstraierPure
       @auxiliary = min
       nil
     end
+    # Set the attribute distinction filter.
+    # `name' specifies the name of an attribute to be distinct.
+    # The return value is always `nil'.
+    def set_distinct(name)
+      Utility::check_types({ name=>String }) if $DEBUG
+      name = name.gsub(/[ \t\r\n\v\f]+/, " ")
+      name = name.strip.squeeze(" ")
+      @distinct = name
+      nil
+    end
     # Set the mask of targets of meta search.
     # `mask' specifies a masking number.  1 means the first target, 2 means the second target, 4
     # means the third target, and power values of 2 and their summation compose the mask.
@@ -349,6 +380,11 @@ module EstraierPure
     def auxiliary()
       @auxiliary
     end
+    # Get the attribute distinction filter.
+    # The return value is the name of the distinct attribute.
+    def distinct()
+      @distinct
+    end
     # Get the mask of targets of meta search.
     # The return value is the mask of targets of meta search.
     def mask()
@@ -367,6 +403,7 @@ module EstraierPure
       @skip = 0
       @options = 0
       @auxiliary = 32
+      @distinct = nil
       @mask = 0
     end
   end
@@ -573,7 +610,7 @@ module EstraierPure
       turl = @url + "/out_doc"
       reqheads = [ "Content-Type: application/x-www-form-urlencoded" ]
       reqheads.push("Authorization: Basic " + Utility::base_encode(@auth)) if @auth
-      reqbody = "uri=" + URI::encode(uri)
+      reqbody = "uri=" + CGI::escape(uri)
       rv = Utility::shuttle_url(turl, @pxhost, @pxport, @timeout, reqheads, reqbody, nil, nil)
       @status = rv
       rv == 200
@@ -620,7 +657,7 @@ module EstraierPure
       turl = @url + "/get_doc"
       reqheads = [ "Content-Type: application/x-www-form-urlencoded" ]
       reqheads.push("Authorization: Basic " + Utility::base_encode(@auth)) if @auth
-      reqbody = "uri=" + URI::encode(uri)
+      reqbody = "uri=" + CGI::escape(uri)
       resbody = StringIO::new
       rv = Utility::shuttle_url(turl, @pxhost, @pxport, @timeout, reqheads, reqbody, nil, resbody)
       @status = rv
@@ -638,7 +675,7 @@ module EstraierPure
       turl = @url + "/get_doc_attr"
       reqheads = [ "Content-Type: application/x-www-form-urlencoded" ]
       reqheads.push("Authorization: Basic " + Utility::base_encode(@auth)) if @auth
-      reqbody = "id=" + id.to_s + "&attr=" + URI::encode(name)
+      reqbody = "id=" + id.to_s + "&attr=" + CGI::escape(name)
       resbody = StringIO::new
       rv = Utility::shuttle_url(turl, @pxhost, @pxport, @timeout, reqheads, reqbody, nil, resbody)
       @status = rv
@@ -656,7 +693,7 @@ module EstraierPure
       turl = @url + "/get_doc_attr"
       reqheads = [ "Content-Type: application/x-www-form-urlencoded" ]
       reqheads.push("Authorization: Basic " + Utility::base_encode(@auth)) if @auth
-      reqbody = "uri=" + URI::encode(uri) + "&attr=" + URI::encode(name)
+      reqbody = "uri=" + CGI::escape(uri) + "&attr=" + CGI::escape(name)
       resbody = StringIO::new
       rv = Utility::shuttle_url(turl, @pxhost, @pxport, @timeout, reqheads, reqbody, nil, resbody)
       @status = rv
@@ -699,7 +736,7 @@ module EstraierPure
       turl = @url + "/etch_doc"
       reqheads = [ "Content-Type: application/x-www-form-urlencoded" ]
       reqheads.push("Authorization: Basic " + Utility::base_encode(@auth)) if @auth
-      reqbody = "uri=" + URI::encode(uri)
+      reqbody = "uri=" + CGI::escape(uri)
       resbody = StringIO::new
       rv = Utility::shuttle_url(turl, @pxhost, @pxport, @timeout, reqheads, reqbody, nil, resbody)
       @status = rv
@@ -723,7 +760,7 @@ module EstraierPure
       turl = @url + "/uri_to_id"
       reqheads = [ "Content-Type: application/x-www-form-urlencoded" ]
       reqheads.push("Authorization: Basic " + Utility::base_encode(@auth)) if @auth
-      reqbody = "uri=" + URI::encode(uri)
+      reqbody = "uri=" + CGI::escape(uri)
       resbody = StringIO::new
       rv = Utility::shuttle_url(turl, @pxhost, @pxport, @timeout, reqheads, reqbody, nil, resbody)
       @status = rv
@@ -901,7 +938,7 @@ module EstraierPure
       turl = @url + "/_set_user"
       reqheads = [ "Content-Type: application/x-www-form-urlencoded" ]
       reqheads.push("Authorization: Basic " + Utility::base_encode(@auth)) if @auth
-      reqbody = "name=" + URI::encode(name) + "&mode=" + mode.to_s
+      reqbody = "name=" + CGI::escape(name) + "&mode=" + mode.to_s
       rv = Utility::shuttle_url(turl, @pxhost, @pxport, @timeout, reqheads, reqbody, nil, nil)
       @status = rv
       rv == 200
@@ -918,7 +955,7 @@ module EstraierPure
       turl = @url + "/_set_link"
       reqheads = [ "Content-Type: application/x-www-form-urlencoded" ]
       reqheads.push("Authorization: Basic " + Utility::base_encode(@auth)) if @auth
-      reqbody = "url=" + URI::encode(url) + "&label=" + label
+      reqbody = "url=" + CGI::escape(url) + "&label=" + label
       reqbody += "&credit=" + credit.to_s if credit >= 0
       rv = Utility::shuttle_url(turl, @pxhost, @pxport, @timeout, reqheads, reqbody, nil, nil)
       @status = rv
@@ -1107,17 +1144,17 @@ module EstraierPure
       if cond.phrase
         buf.write("&") if buf.length > 0
         buf.write("phrase=")
-        buf.write(URI::encode(cond.phrase))
+        buf.write(CGI::escape(cond.phrase))
       end
       for i in 0...cond.attrs.length
         buf.write("&") if buf.length > 0
         buf.write("attr" + (i + 1).to_s + "=")
-        buf.write(URI::encode(cond.attrs[i]))
+        buf.write(CGI::escape(cond.attrs[i]))
       end
       if cond.order
         buf.write("&") if buf.length > 0
         buf.write("order=")
-        buf.write(URI::encode(cond.order))
+        buf.write(CGI::escape(cond.order))
       end
       if cond.max >= 0
         buf.write("&") if buf.length > 0
@@ -1128,6 +1165,10 @@ module EstraierPure
       end
       buf.write("&options=" + cond.options.to_s) if cond.options > 0
       buf.write("&auxiliary=" + cond.auxiliary.to_s)
+      if cond.distinct
+        buf.write("&distinct=")
+        buf.write(CGI::escape(cond.distinct))
+      end
       buf.write("&depth=" + depth.to_s) if depth > 0
       buf.write("&wwidth=" + wwidth.to_s)
       buf.write("&hwidth=" + hwidth.to_s)
