@@ -148,14 +148,14 @@ int log_rotate(const char *rootdir, const char *path){
 
 
 /* Initialize the root directory. */
-int waver_init(const char *rootdir){
+int waver_init(const char *rootdir, int options){
   DEPOT *metadb;
   CURIA *trace;
   QUEUE *queue;
   ESTMTDB *index;
   FILE *ofp;
   char path[URIBUFSIZ];
-  int err, ecode;
+  int err, tracebnum, estopts, ecode;
   assert(rootdir);
   if(est_mkdir(rootdir) == -1 && errno != EEXIST) return FALSE;
   err = FALSE;
@@ -208,10 +208,10 @@ int waver_init(const char *rootdir){
     fprintf(ofp, "# regular expressions and replacement strings to normalize URLs\n");
     fprintf(ofp, "replace: ^http://127.0.0.1/{{!}}http://localhost/\n");
     fprintf(ofp, "\n");
-    fprintf(ofp, "# allowing regular expressions of URLs to be visited.\n");
+    fprintf(ofp, "# allowing regular expressions of URLs to be visited\n");
     fprintf(ofp, "allowrx: ^http://\n");
     fprintf(ofp, "\n");
-    fprintf(ofp, "# denying regular expressions of URLs to be visited.\n");
+    fprintf(ofp, "# denying regular expressions of URLs to be visited\n");
     fprintf(ofp, "denyrx: \\.(css|js|csv|tsv|log|md5|crc|conf|ini|inf|lnk|sys|tmp|bak)$\n");
     fprintf(ofp, "denyrx: \\.(xml|xsl|xslt|rdf|rss|dtd|sgml|sgm)$\n");
     fprintf(ofp, "denyrx: \\.(pgp|sig|cer|csr|pem|key|b64|uu|uue|[0-9])$\n");
@@ -226,10 +226,14 @@ int waver_init(const char *rootdir){
     fprintf(ofp, "denyrx: (/core$|/core\\.[0-9]*$|/casket/)\n");
     fprintf(ofp, "denyrx: ://(localhost|[a-z]*\\.localdomain|127\\.0\\.0\\.1)/\n");
     fprintf(ofp, "\n");
-    fprintf(ofp, "# denying regular expressions of URLs to be indexed.\n");
+    fprintf(ofp, "# denying regular expressions of URLs to be indexed\n");
     fprintf(ofp, "noidxrx: /\\?[a-z]=[a-z](;|$)\n");
     fprintf(ofp, "\n");
-    fprintf(ofp, "# media types (regular expressions and filter commands)\n");
+    fprintf(ofp, "# URL rules (regular expressions and media types)\n");
+    fprintf(ofp, "urlrule: \\.est${{!}}text/x-estraier-draft\n");
+    fprintf(ofp, "urlrule: \\.(eml|mime|mht|mhtml)${{!}}message/rfc822\n");
+    fprintf(ofp, "\n");
+    fprintf(ofp, "# media type rules (regular expressions and filter commands)\n");
     fprintf(ofp, "typerule: ^text/x-estraier-draft${{!}}%s\n", DRAFTCMD);
     fprintf(ofp, "typerule: ^text/plain${{!}}%s\n", TEXTCMD);
     fprintf(ofp, "typerule: ^(text/html|application/xhtml+xml)${{!}}%s\n", HTMLCMD);
@@ -286,6 +290,18 @@ int waver_init(const char *rootdir){
   } else {
     err = TRUE;
   }
+  tracebnum = TRACEBNUM;
+  estopts = 0;
+  if(options & WI_SMALL){
+    tracebnum = TRACEBNUM / 2;
+    estopts = ESTDBSMALL;
+  } else if(options & WI_LARGE){
+    tracebnum = TRACEBNUM * 2;
+    estopts = ESTDBLARGE;
+  } else if(options & WI_HUGE){
+    tracebnum = TRACEBNUM * 4;
+    estopts = ESTDBHUGE;
+  }
   sprintf(path, "%s%c%s", rootdir, ESTPATHCHR, QUEUEFILE);
   unlink(path);
   if((queue = queue_open(path)) != NULL){
@@ -294,13 +310,13 @@ int waver_init(const char *rootdir){
     err = TRUE;
   }
   sprintf(path, "%s%c%s", rootdir, ESTPATHCHR, TRACEFILE);
-  if((trace = cropen(path, CR_OWRITER | CR_OCREAT | CR_OTRUNC, TRACEBNUM, TRACEDNUM)) != NULL){
+  if((trace = cropen(path, CR_OWRITER | CR_OCREAT | CR_OTRUNC, tracebnum, TRACEDNUM)) != NULL){
     if(!crclose(trace)) err = TRUE;
   } else {
     err = TRUE;
   }
   sprintf(path, "%s%c%s", rootdir, ESTPATHCHR, INDEXDIR);
-  if((index = est_mtdb_open(path, ESTDBWRITER | ESTDBCREAT | ESTDBTRUNC | ESTDBHUGE,
+  if((index = est_mtdb_open(path, ESTDBWRITER | ESTDBCREAT | ESTDBTRUNC | estopts,
                             &ecode)) != NULL){
     if(!est_mtdb_add_attr_index(index, ESTDATTRURI, ESTIDXATTRSTR)) err = TRUE;
     if(!est_mtdb_add_attr_index(index, ESTDATTRTITLE, ESTIDXATTRSTR)) err = TRUE;
@@ -326,6 +342,7 @@ WAVER *waver_open(const char *rootdir){
   WAVER *waver;
   UNRULE unrule;
   PMRULE pmrule;
+  URLRULE urlrule;
   MTRULE mtrule;
   DEPOT *metadb;
   CURIA *trace;
@@ -378,6 +395,7 @@ WAVER *waver_open(const char *rootdir){
   waver->queuesize = INT_MAX;
   waver->unrules = cblistopen();
   waver->pmrules = cblistopen();
+  waver->urlrules = cblistopen();
   waver->mtrules = cblistopen();
   waver->language = ESTLANGEN;
   waver->textlimit = 0;
@@ -465,6 +483,16 @@ WAVER *waver_open(const char *rootdir){
         cblistpush(waver->pmrules, (char *)&pmrule, sizeof(PMRULE));
       }
       free(tmp);
+    } else if(cbstrfwimatch(rp, "urlrule:")){
+      rp = skiplabel(rp);
+      if((pv = strstr(rp, "{{!}}")) != NULL){
+        tmp = cbmemdup(rp, pv - rp);
+        if((urlrule.regex = est_regex_new(tmp)) != NULL){
+          urlrule.type = cbmemdup(pv + 5, -1);
+          cblistpush(waver->urlrules, (char *)&urlrule, sizeof(URLRULE));
+        }
+        free(tmp);
+      }
     } else if(cbstrfwimatch(rp, "typerule:")){
       rp = skiplabel(rp);
       if((pv = strstr(rp, "{{!}}")) != NULL){
@@ -563,6 +591,7 @@ WAVER *waver_open(const char *rootdir){
 int waver_close(WAVER *waver){
   UNRULE *unrule;
   PMRULE *pmrule;
+  URLRULE *urlrule;
   MTRULE *mtrule;
   ESTNODE *node;
   const char *kbuf;
@@ -584,6 +613,12 @@ int waver_close(WAVER *waver){
     free(mtrule->filter);
   }
   cblistclose(waver->mtrules);
+  for(i = 0; i < cblistnum(waver->urlrules); i++){
+    urlrule = (URLRULE *)cblistval(waver->urlrules, i, NULL);
+    est_regex_delete(urlrule->regex);
+    free(urlrule->type);
+  }
+  cblistclose(waver->urlrules);
   for(i = 0; i < cblistnum(waver->pmrules); i++){
     pmrule = (PMRULE *)cblistval(waver->pmrules, i, NULL);
     est_regex_delete(pmrule->regex);
@@ -830,8 +865,10 @@ void kwords_reduce(CBMAP *kwords, int num, int fadeout){
 
 /* Fetch a document of a URL. */
 int fetch_document(const char *url, const char *pxhost, int pxport, int outsec, time_t mdate,
-                   const CBLIST *mtrules, int *codep, CBDATUM *raw, CBMAP *heads,
+                   const CBLIST *urlrules, const CBLIST *mtrules,
+                   int *codep, CBDATUM *raw, CBMAP *heads,
                    CBLIST *links, const CBLIST *unrules, ESTDOC *doc, int lang){
+  URLRULE *urlrule;
   MTRULE *mtrule;
   UNRULE *unrule;
   ESTDOC *rdoc;
@@ -898,6 +935,15 @@ int fetch_document(const char *url, const char *pxhost, int pxport, int outsec,
     if(codep) *codep = rescode;
     return FALSE;
   }
+  if(urlrules){
+    for(i = 0; i < cblistnum(urlrules); i++){
+      urlrule = (URLRULE *)cblistval(urlrules, i, NULL);
+      if(est_regex_match(urlrule->regex, url)){
+        cbmapput(heads, "content-type", -1, urlrule->type, -1, TRUE);
+        break;
+      }
+    }
+  }
   if(!(vbuf = cbmapget(heads, "content-type", -1, NULL))) vbuf = "text/plain";
   type = cbmemdup(vbuf, -1);
   if((pv = strchr(type, ';')) != NULL) *pv = '\0';
@@ -1366,6 +1412,22 @@ static void make_doc_from_html(const char *buf, int size, const char *penc, int
            !cbstrfwimatch(value, "mailto:") && !cbstrfwimatch(value, "javascript:"))
           cblistpush(links, value, -1);
         cbmapclose(attrs);
+      } else if(links && cbstrfwimatch(elem, "<object")){
+        attrs = cbxmlattrs(elem);
+        value = cbmapget(attrs, "data", -1, NULL);
+        if(!value) value = cbmapget(attrs, "DATA", -1, NULL);
+        if(value && !cbstrfwimatch(value, "https:") && !cbstrfwimatch(value, "ftp:") &&
+           !cbstrfwimatch(value, "mailto:") && !cbstrfwimatch(value, "javascript:"))
+          cblistpush(links, value, -1);
+        cbmapclose(attrs);
+      } else if(links && (cbstrfwimatch(elem, "<embed") || cbstrfwimatch(elem, "<iframe"))){
+        attrs = cbxmlattrs(elem);
+        value = cbmapget(attrs, "src", -1, NULL);
+        if(!value) value = cbmapget(attrs, "SRC", -1, NULL);
+        if(value && !cbstrfwimatch(value, "https:") && !cbstrfwimatch(value, "ftp:") &&
+           !cbstrfwimatch(value, "mailto:") && !cbstrfwimatch(value, "javascript:"))
+          cblistpush(links, value, -1);
+        cbmapclose(attrs);
       }
     } else {
       CB_DATUMCAT(datum, " ", 1);
@@ -1700,7 +1762,7 @@ static void make_doc_from_mime(const char *buf, int size, const char *penc, int
         CB_DATUMCLOSE(datum);
         CB_LISTCLOSE(lines);
       }
-    } else if(cbstrfwimatch(key, "text/html")){
+    } else if(cbstrfwimatch(key, "text/html") || cbstrfwimatch(key, "application/xhtml+xml")){
       tdoc = est_doc_new();
       make_doc_from_html(body, bsiz, penc, plang, tdoc, links);
       if((text = est_doc_attr(tdoc, ESTDATTRTITLE)) != NULL){
@@ -1777,18 +1839,22 @@ static void make_doc_with_xcmd(const char *xcmd, const char *url, const char *bu
   const char *tmpdir, *pv, *ext, *fmt;
   char iname[URIBUFSIZ], oname[URIBUFSIZ], cmd[URIBUFSIZ];
   char *rbuf, numbuf[NUMBUFSIZ];
-  int rsiz, rnd, pid;
+  int rnd, pid, rsiz;
   struct stat sbuf;
   assert(buf && size >= 0 && url && xcmd);
   if(ESTPATHCHR == '/' && stat("/tmp", &sbuf) == 0){
     tmpdir = "/tmp";
+  } else if(ESTPATHCHR == '\\' &&
+            ((pv = getenv("TMP")) != NULL || (pv = getenv("TEMP")) != NULL) &&
+            stat(pv, &sbuf) == 0){
+    tmpdir = pv;
   } else {
     tmpdir = ESTCDIRSTR;
   }
   ext = NULL;
   if((pv = strrchr(url, ESTPATHCHR)) != NULL) url = pv;
   if((pv = strrchr(url, ESTEXTCHR)) != NULL) ext = pv;
-  if(!ext || strlen(ext) >= 32) ext = "";
+  if(!ext || strlen(ext) >= 32 || strchr(ext, '"') || strchr(ext, '\\')) ext = "";
   rnd = dpouterhash(url, -1) & 0xffff;
   pid = (int)getpid() & 0xffff;
   sprintf(iname, "%s%cxcmd-in-%04X%04X%s", tmpdir, ESTPATHCHR, pid, rnd, ext);
@@ -1804,16 +1870,18 @@ static void make_doc_with_xcmd(const char *xcmd, const char *url, const char *bu
     fmt = MIMECMD;
     xcmd += 2;
   }
-  sprintf(cmd, "%s \"%s\" \"%s\"", xcmd, iname, oname);
   cbwritefile(iname, buf, size);
+  sprintf(cmd, "%s \"%s\" \"%s\"", xcmd, iname, oname);
   system(cmd);
   if((rbuf = cbreadfile(oname, &rsiz)) != NULL){
-    if(fmt == TEXTCMD){
+    if(fmt == DRAFTCMD){
+      make_doc_from_draft(rbuf, rsiz, doc, links);
+    } else if(fmt == TEXTCMD){
       make_doc_from_text(rbuf, rsiz, penc, plang, doc, links);
     } else if(fmt == HTMLCMD){
       make_doc_from_html(rbuf, rsiz, penc, plang, doc, links);
-    } else if(fmt == DRAFTCMD){
-      make_doc_from_draft(rbuf, rsiz, doc, links);
+    } else if(fmt == MIMECMD){
+      make_doc_from_mime(rbuf, rsiz, penc, plang, doc, links);
     }
     free(rbuf);
   }
