@@ -51,7 +51,7 @@
 #define QUEUENCN       512               /* number of non-leaf cache of the queue */
 
 #define TRACEFILE      "_trace"          /* name of the trace file */
-#define TRACEBNUM      982981            /* bucket number of the trace database */
+#define TRACEBNUM      425977            /* bucket number of the trace database */
 #define TRACEDNUM      3                 /* division number of the trace database */
 
 #define DATTRDEPTH     "_depth"          /* name of the attribute of the depth */
@@ -60,6 +60,12 @@
 #define HTMLCMD        "[HTML]"          /* built-in command for HTML */
 #define MIMECMD        "[MIME]"          /* built-in command for MIME */
 
+enum {                                   /* enumeration for initializing options */
+  WI_SMALL = 1 << 0,                     /* small scale */
+  WI_LARGE = 1 << 1,                     /* large scale */
+  WI_HUGE = 1 << 2                       /* huge scale */
+};
+
 enum {                                   /* enumeration for running modes */
   LL_DEBUG = 1,                          /* debug */
   LL_INFO = 2,                           /* information */
@@ -100,7 +106,12 @@ typedef struct {                         /* type of structure for a permission *
   int index;                             /* to be indexed */
 } PMRULE;
 
-typedef struct {                         /* type of structure for a media type */
+typedef struct {                         /* type of structure for a URL rule */
+  void *regex;                           /* regular expressions */
+  char *type;                            /* media type */
+} URLRULE;
+
+typedef struct {                         /* type of structure for a media type rule */
   void *regex;                           /* regular expressions */
   char *filter;                          /* filter command */
 } MTRULE;
@@ -126,6 +137,7 @@ typedef struct {                         /* type of structure for waver */
   int queuesize;                         /* size of the priority queue */
   CBLIST *unrules;                       /* rules of URL normalization */
   CBLIST *pmrules;                       /* rules of permission */
+  CBLIST *urlrules;                      /* rules of URL */
   CBLIST *mtrules;                       /* rules of media types */
   int language;                          /* preferred language */
   int textlimit;                         /* text size limitation */
@@ -170,8 +182,9 @@ void log_print(int level, const char *format, ...);
 
 /* Initialize the root directory.
    `rootdir' specifies the path of the root directory.
+   `options' specifies the options: `WI_SMALL', `WI_LARGE', or `WI_HUGE'.
    The return value is true if success, else it is false. */
-int waver_init(const char *rootdir);
+int waver_init(const char *rootdir, int options);
 
 
 /* Open a waver handle.
@@ -282,6 +295,8 @@ void kwords_reduce(CBMAP *kwords, int num, int fadeout);
    `pxport' specifies the port number of the proxy.
    `outsec' specifies timeout in seconds.  If it is negative, it is not used.
    `mdate' specifies the last-modified date.  If it is not more than 0, it is not used.
+   `urlrules' specifies a list object conteining type rules of URLs.  If it is `NULL', the
+   default rule is applied.
    `mtrules' specifies a list object conteining filter rules of media types.  If it is `NULL',
    the default rule is applied.
    `codep' specifies the pointer to a variable to which the status code of respnese is
@@ -296,7 +311,8 @@ void kwords_reduce(CBMAP *kwords, int num, int fadeout);
    `lang' specifies the code of preferred language.
    The return value is true if success, else it is false. */
 int fetch_document(const char *url, const char *pxhost, int pxport, int outsec, time_t mdate,
-                   const CBLIST *mtrules, int *codep, CBDATUM *raw, CBMAP *heads,
+                   const CBLIST *urlrules, const CBLIST *mtrules,
+                   int *codep, CBDATUM *raw, CBMAP *heads,
                    CBLIST *links, const CBLIST *unrules, ESTDOC *doc, int lang);
 
 
